@@ -10,19 +10,10 @@ GET_PIP_CACHE_LOCATION=/opt/stack/cache/files/get-pip.py
 
 function octavia_install {
     if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
-        setup_develop $OCTAVIA_DIR redis
+        setup_develop $OCTAVIA_DIR ${OCTAVIA_JOBBOARD_BACKEND}
     else
         setup_develop $OCTAVIA_DIR
     fi
-    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
-        if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-            if [[ ${DISTRO} =~ (rhel|centos) ]]; then
-                install_package qemu-kvm
-            else
-                install_package qemu
-            fi
-        fi
-    fi
 }
 
 function octaviaclient_install {
@@ -134,6 +125,8 @@ function build_octavia_worker_image {
         fi
         sudo mkdir -m755 ${dib_logs}
         sudo chown $STACK_USER ${dib_logs}
+        # Workaround for rockylinux images
+        export DIB_CONTAINERFILE_RUNTIME_ROOT=1
         $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-} ${PARAM_OCTAVIA_AMP_IMAGE_ARCH:-} ${PARAM_OCTAVIA_AMP_DISABLE_TMP_FS:-} ${PARAM_OCTAVIA_AMP_ENABLE_FIPS:-}
     fi
 
@@ -289,10 +282,6 @@ function octavia_configure {
     sudo mkdir -m 700 -p $OCTAVIA_RUN_DIR
     safe_chown $STACK_USER $OCTAVIA_RUN_DIR
 
-    if ! [ -e $OCTAVIA_CONF ] ; then
-        cp $OCTAVIA_DIR/etc/octavia.conf $OCTAVIA_CONF
-    fi
-
     if ! [ -e $OCTAVIA_AUDIT_MAP ] ; then
         cp $OCTAVIA_DIR/etc/audit/octavia_api_audit_map.conf.sample $OCTAVIA_AUDIT_MAP
     fi
@@ -310,6 +299,11 @@ function octavia_configure {
         iniset $OCTAVIA_CONF task_flow persistence_connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia_persistence"
         iniset $OCTAVIA_CONF task_flow jobboard_expiration_time ${OCTAVIA_JOBBOARD_EXPIRATION_TIME}
         iniset $OCTAVIA_CONF task_flow jobboard_enabled True
+        if [[ ${OCTAVIA_JOBBOARD_BACKEND} == "etcd" ]]; then
+            iniset $OCTAVIA_CONF task_flow jobboard_backend_driver etcd_taskflow_driver
+            iniset $OCTAVIA_CONF task_flow jobboard_backend_port 2379
+            iniset $OCTAVIA_CONF task_flow jobboard_backend_hosts ${SERVICE_HOST}
+        fi
     fi
     # Configure keystone auth_token for all users
     configure_keystone_authtoken_middleware $OCTAVIA_CONF octavia
@@ -325,6 +319,16 @@ function octavia_configure {
     iniset $OCTAVIA_CONF service_auth cafile $SSL_BUNDLE_FILE
     iniset $OCTAVIA_CONF service_auth memcached_servers $SERVICE_HOST:11211
 
+    # neutron
+    iniset $OCTAVIA_CONF neutron auth_url $KEYSTONE_SERVICE_URI
+    iniset $OCTAVIA_CONF neutron auth_type password
+    iniset $OCTAVIA_CONF neutron username $OCTAVIA_USERNAME
+    iniset $OCTAVIA_CONF neutron password $OCTAVIA_PASSWORD
+    iniset $OCTAVIA_CONF neutron user_domain_name $OCTAVIA_USER_DOMAIN_NAME
+    iniset $OCTAVIA_CONF neutron project_name $OCTAVIA_PROJECT_NAME
+    iniset $OCTAVIA_CONF neutron project_domain_name $OCTAVIA_PROJECT_DOMAIN_NAME
+    iniset $OCTAVIA_CONF neutron cafile $SSL_BUNDLE_FILE
+
     # Setting other required default options
     iniset $OCTAVIA_CONF controller_worker amphora_driver ${OCTAVIA_AMPHORA_DRIVER}
     iniset $OCTAVIA_CONF controller_worker compute_driver ${OCTAVIA_COMPUTE_DRIVER}
@@ -340,7 +344,7 @@ function octavia_configure {
     iniset $OCTAVIA_CONF house_keeping amphora_expiry_age ${OCTAVIA_AMP_EXPIRY_AGE}
     iniset $OCTAVIA_CONF house_keeping load_balancer_expiry_age ${OCTAVIA_LB_EXPIRY_AGE}
 
-    iniset $OCTAVIA_CONF DEFAULT transport_url $(get_transport_url)
+    iniset_rpc_backend octavia $OCTAVIA_CONF
 
     iniset $OCTAVIA_CONF oslo_messaging rpc_thread_pool_size 2
     iniset $OCTAVIA_CONF oslo_messaging topic octavia_prov
@@ -427,13 +431,18 @@ function octavia_configure {
     # Controller side symmetric encryption, not used for PKI
     iniset $OCTAVIA_CONF certificates server_certs_key_passphrase insecure-key-do-not-use-this-key
 
+    if [[ "$OCTAVIA_USE_ADVANCED_RBAC" == "True" ]]; then
+        cp $OCTAVIA_DIR/etc/policy/octavia-advanced-rbac-policy.yaml $OCTAVIA_CONF_DIR/policy.yaml
+        iniset $OCTAVIA_CONF oslo_policy policy_file $OCTAVIA_CONF_DIR/policy.yaml
+    fi
     if [[ "$OCTAVIA_USE_LEGACY_RBAC" == "True" ]]; then
         cp $OCTAVIA_DIR/etc/policy/admin_or_owner-policy.yaml $OCTAVIA_CONF_DIR/policy.yaml
+        iniset $OCTAVIA_CONF oslo_policy policy_file $OCTAVIA_CONF_DIR/policy.yaml
+    fi
+    if [[ "$OCTAVIA_USE_KEYSTONE_DEFAULT_ROLES" == "True" ]]; then
+        cp $OCTAVIA_DIR/etc/policy/keystone_default_roles-policy.yaml $OCTAVIA_CONF_DIR/policy.yaml
+        iniset $OCTAVIA_CONF oslo_policy policy_file $OCTAVIA_CONF_DIR/policy.yaml
     fi
-
-    # create dhclient.conf file for dhclient
-    sudo mkdir -m755 -p $OCTAVIA_DHCLIENT_DIR
-    sudo cp $OCTAVIA_DIR/etc/dhcp/dhclient.conf $OCTAVIA_DHCLIENT_CONF
 
     if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
         if [[ "$WSGI_MODE" == "uwsgi" ]]; then
@@ -479,8 +488,20 @@ function create_mgmt_network_interface {
     fi
     sudo ip link set dev o-hm0 address $MGMT_PORT_MAC
 
+    function _get_firewall () {
+        # The devstack CI forces the use of iptables, the openstack-INPUT table
+        # can be used to indicate it.
+        if sudo iptables -L -n -v | grep openstack-INPUT; then
+            echo "iptables"
+        elif [[ -x $(which nft 2> /dev/null) ]]; then
+            echo "nft"
+        else
+            echo "iptables"
+        fi
+    }
+
     # Check if the host is using nftables, an alternative to iptables
-    if [ -x "$(sudo bash -c 'command -v nft')" ]; then
+    if [[ $(_get_firewall) == "nft" ]]; then
         sudo nft add table inet octavia
         sudo nft add chain inet octavia o-hm0-incoming { type filter hook input priority 0\;}
         sudo nft flush chain inet octavia o-hm0-incoming
@@ -529,12 +550,9 @@ function build_mgmt_network {
     # Create network and attach a subnet
     openstack network create lb-mgmt-net
     if [ $SERVICE_IP_VERSION == '6' ] ; then
-        openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET_IPV6 --allocation-pool start=$OCTAVIA_MGMT_SUBNET_IPV6_START,end=$OCTAVIA_MGMT_SUBNET_IPV6_END --network lb-mgmt-net --ip-version 6 --ipv6-address-mode slaac --ipv6-ra-mode slaac lb-mgmt-subnet
-        # SLAAC needs a router on the subnet to advertise the prefix.
-        openstack router create lb-mgmt-router
-        openstack router add subnet lb-mgmt-router lb-mgmt-subnet
+        openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET_IPV6 --allocation-pool start=$OCTAVIA_MGMT_SUBNET_IPV6_START,end=$OCTAVIA_MGMT_SUBNET_IPV6_END --network lb-mgmt-net --ip-version 6 --gateway none --no-dhcp lb-mgmt-subnet
     else
-        openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet
+        openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net --gateway none --no-dhcp lb-mgmt-subnet
     fi
 
     # Create security group and rules
@@ -612,15 +630,19 @@ function configure_rsyslog {
 
 function octavia_start {
 
-    if  ! ps aux | grep -q [o]-hm0 && [ $OCTAVIA_NODE != 'api' ] ; then
+    if  [ $OCTAVIA_NODE != 'api' ] ; then
+        # This is probably out of scope here? Load it from config
+        MGMT_PORT_IP=$(iniget $OCTAVIA_CONF health_manager bind_ip)
+
         if [ $SERVICE_IP_VERSION == '6' ] ; then
-                # This is probably out of scope here? Load it from config
-                MGMT_PORT_IP=$(iniget $OCTAVIA_CONF health_manager bind_ip)
-            sudo ip addr add $MGMT_PORT_IP/64 dev o-hm0
-            sudo ip link set o-hm0 up
+            MGMT_SUBNET_ARRAY=(${OCTAVIA_MGMT_SUBNET_IPV6//// })
         else
-            sudo dhclient -v o-hm0 -cf $OCTAVIA_DHCLIENT_CONF
+            MGMT_SUBNET_ARRAY=(${OCTAVIA_MGMT_SUBNET//// })
         fi
+        MGMT_SUBNET_MASK=${MGMT_SUBNET_ARRAY[1]}
+
+        sudo ip addr add $MGMT_PORT_IP/$MGMT_SUBNET_MASK dev o-hm0
+        sudo ip link set o-hm0 up
     fi
 
     if [ $OCTAVIA_NODE == 'main' ]; then
@@ -653,21 +675,31 @@ function octavia_stop {
     stop_process $OCTAVIA_HOUSEKEEPER
     stop_process $OCTAVIA_HEALTHMANAGER
 
+    # TODO(johnsom) Remove this in 2025.2 release so upgrades will stop this
+    # process.
     # Kill dhclient process started for o-hm0 interface
     pids=$(ps aux | awk '/[o]-hm0/ { print $2 }')
     [ ! -z "$pids" ] && sudo kill $pids
+
     if function_exists octavia_delete_network_interface_device ; then
         octavia_delete_network_interface_device o-hm0
-    elif [[ $NEUTRON_AGENT == "openvswitch" || $Q_AGENT == "openvswitch" ]]; then
-        # This elif can go away in the X cycle, needed for grenade old/new logic
-        :  # Do nothing
-    elif [[ $NEUTRON_AGENT == "linuxbridge" || $Q_AGENT == "linuxbridge" ]]; then
+    fi
+
+    # Grenade upgrades need the IP address removed here
+    MGMT_PORT_IP=$(iniget $OCTAVIA_CONF health_manager bind_ip)
+    if [ $SERVICE_IP_VERSION == '6' ] ; then
+        MGMT_SUBNET_ARRAY=(${OCTAVIA_MGMT_SUBNET_IPV6//// })
+    else
+        MGMT_SUBNET_ARRAY=(${OCTAVIA_MGMT_SUBNET//// })
+    fi
+    MGMT_SUBNET_MASK=${MGMT_SUBNET_ARRAY[1]}
+    sudo ip addr del $MGMT_PORT_IP/$MGMT_SUBNET_MASK dev o-hm0
+
+    if [[ $NEUTRON_AGENT == "linuxbridge" || $Q_AGENT == "linuxbridge" ]]; then
         # This elif can go away in the X cycle, needed for grenade old/new logic
         if ip link show o-hm0 ; then
             sudo ip link del o-hm0
         fi
-    else
-        die "Unknown network controller. Please define octavia_delete_network_interface_device"
     fi
 
     if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
@@ -719,12 +751,12 @@ function octavia_cleanup {
 }
 
 function add_load-balancer_roles {
-    openstack role create load-balancer_observer
-    openstack role create load-balancer_global_observer
-    openstack role create load-balancer_member
-    openstack role create load-balancer_admin
-    openstack role create load-balancer_quota_admin
-    openstack role add --user demo --project demo load-balancer_member
+    get_or_create_role load-balancer_observer
+    get_or_create_role load-balancer_global_observer
+    get_or_create_role load-balancer_member
+    get_or_create_role load-balancer_admin
+    get_or_create_role load-balancer_quota_admin
+    get_or_add_user_project_role load-balancer_member demo demo
 }
 
 function octavia_init {
