@@ -16,6 +16,7 @@ import os
 import stat
 
 import flask
+from jsonschema import validate
 from oslo_config import cfg
 from oslo_log import log as logging
 import webob
@@ -25,11 +26,15 @@ from octavia.amphorae.backends.agent import api_server
 from octavia.amphorae.backends.agent.api_server import amphora_info
 from octavia.amphorae.backends.agent.api_server import certificate_update
 from octavia.amphorae.backends.agent.api_server import keepalived
+from octavia.amphorae.backends.agent.api_server import keepalivedlvs
 from octavia.amphorae.backends.agent.api_server import loadbalancer
 from octavia.amphorae.backends.agent.api_server import osutils
 from octavia.amphorae.backends.agent.api_server import plug
-from octavia.amphorae.backends.agent.api_server import udp_listener_base
+from octavia.amphorae.backends.agent.api_server import rules_schema
 from octavia.amphorae.backends.agent.api_server import util
+from octavia.amphorae.backends.utils import nftable_utils
+from octavia.common import constants as consts
+
 
 BUFFER = 1024
 CONF = cfg.CONF
@@ -50,37 +55,42 @@ def register_app_error_handler(app):
         app.register_error_handler(code, make_json_error)
 
 
-class Server(object):
+class Server:
     def __init__(self):
         self.app = flask.Flask(__name__)
         self._osutils = osutils.BaseOS.get_os_util()
         self._keepalived = keepalived.Keepalived()
         self._loadbalancer = loadbalancer.Loadbalancer()
-        self._udp_listener = (udp_listener_base.UdpListenerApiServerBase.
-                              get_server_driver())
+        self._lvs_listener = keepalivedlvs.KeepalivedLvs()
         self._plug = plug.Plug(self._osutils)
         self._amphora_info = amphora_info.AmphoraInfo(self._osutils)
 
         register_app_error_handler(self.app)
 
+        self._plug.plug_lo()
+
         self.app.add_url_rule(rule='/', view_func=self.version_discovery,
                               methods=['GET'])
         self.app.add_url_rule(rule=PATH_PREFIX +
                               '/loadbalancer/<amphora_id>/<lb_id>/haproxy',
                               view_func=self.upload_haproxy_config,
                               methods=['PUT'])
+        # TODO(gthiemonge) rename 'udp_listener' endpoint to 'lvs_listener'
+        # when api_version is bumped
         self.app.add_url_rule(rule=PATH_PREFIX +
                               '/listeners/<amphora_id>/<listener_id>'
                               '/udp_listener',
-                              view_func=self.upload_udp_listener_config,
+                              view_func=self.upload_lvs_listener_config,
                               methods=['PUT'])
         self.app.add_url_rule(rule=PATH_PREFIX +
                               '/loadbalancer/<lb_id>/haproxy',
                               view_func=self.get_haproxy_config,
                               methods=['GET'])
+        # TODO(gthiemonge) rename 'udp_listener' endpoint to 'lvs_listener'
+        # when api_version is bumped
         self.app.add_url_rule(rule=PATH_PREFIX +
                               '/listeners/<listener_id>/udp_listener',
-                              view_func=self.get_udp_listener_config,
+                              view_func=self.get_lvs_listener_config,
                               methods=['GET'])
         self.app.add_url_rule(rule=PATH_PREFIX +
                               '/loadbalancer/<object_id>/<action>',
@@ -130,44 +140,47 @@ class Server(object):
         self.app.add_url_rule(rule=PATH_PREFIX + '/interface/<ip_addr>',
                               view_func=self.get_interface,
                               methods=['GET'])
+        self.app.add_url_rule(rule=PATH_PREFIX + '/interface/<ip_addr>/rules',
+                              view_func=self.set_interface_rules,
+                              methods=['PUT'])
 
     def upload_haproxy_config(self, amphora_id, lb_id):
         return self._loadbalancer.upload_haproxy_config(amphora_id, lb_id)
 
-    def upload_udp_listener_config(self, amphora_id, listener_id):
-        return self._udp_listener.upload_udp_listener_config(listener_id)
+    def upload_lvs_listener_config(self, amphora_id, listener_id):
+        return self._lvs_listener.upload_lvs_listener_config(listener_id)
 
     def get_haproxy_config(self, lb_id):
         return self._loadbalancer.get_haproxy_config(lb_id)
 
-    def get_udp_listener_config(self, listener_id):
-        return self._udp_listener.get_udp_listener_config(listener_id)
+    def get_lvs_listener_config(self, listener_id):
+        return self._lvs_listener.get_lvs_listener_config(listener_id)
 
     def start_stop_lb_object(self, object_id, action):
-        protocol = util.get_protocol_for_lb_object(object_id)
-        if protocol == 'UDP':
-            return self._udp_listener.manage_udp_listener(
+        backend = util.get_backend_for_lb_object(object_id)
+        if backend == consts.LVS_BACKEND:
+            return self._lvs_listener.manage_lvs_listener(
                 listener_id=object_id, action=action)
         return self._loadbalancer.start_stop_lb(lb_id=object_id, action=action)
 
     def delete_lb_object(self, object_id):
-        protocol = util.get_protocol_for_lb_object(object_id)
-        if protocol == 'UDP':
-            return self._udp_listener.delete_udp_listener(object_id)
+        backend = util.get_backend_for_lb_object(object_id)
+        if backend == consts.LVS_BACKEND:
+            return self._lvs_listener.delete_lvs_listener(object_id)
         return self._loadbalancer.delete_lb(object_id)
 
     def get_details(self):
         return self._amphora_info.compile_amphora_details(
-            extend_udp_driver=self._udp_listener)
+            extend_lvs_driver=self._lvs_listener)
 
     def get_info(self):
         return self._amphora_info.compile_amphora_info(
-            extend_udp_driver=self._udp_listener)
+            extend_lvs_driver=self._lvs_listener)
 
     def get_all_listeners_status(self):
-        udp_listeners = self._udp_listener.get_all_udp_listeners_status()
+        lvs_listeners = self._lvs_listener.get_all_lvs_listeners_status()
         return self._loadbalancer.get_all_listeners_status(
-            other_listeners=udp_listeners)
+            other_listeners=lvs_listeners)
 
     def upload_certificate(self, lb_id, filename):
         return self._loadbalancer.upload_certificate(lb_id, filename)
@@ -195,7 +208,9 @@ class Server(object):
                                    net_info['mac_address'],
                                    net_info.get('mtu'),
                                    net_info.get('vrrp_ip'),
-                                   net_info.get('host_routes'))
+                                   net_info.get('host_routes', ()),
+                                   net_info.get('additional_vips', ()),
+                                   net_info.get('is_sriov', False))
 
     def plug_network(self):
         try:
@@ -207,7 +222,9 @@ class Server(object):
                 description='Invalid port information') from e
         return self._plug.plug_network(port_info['mac_address'],
                                        port_info.get('fixed_ips'),
-                                       port_info.get('mtu'))
+                                       port_info.get('mtu'),
+                                       port_info.get('vip_net_info'),
+                                       port_info.get('is_sriov'))
 
     def upload_cert(self):
         return certificate_update.upload_server_cert()
@@ -239,11 +256,31 @@ class Server(object):
         except Exception as e:
             LOG.error("Unable to update amphora-agent configuration: %s",
                       str(e))
-            return webob.Response(json=dict(
-                message="Unable to update amphora-agent configuration.",
-                details=str(e)), status=500)
+            return webob.Response(json={
+                'message': "Unable to update amphora-agent configuration.",
+                'details': str(e)}, status=500)
 
         return webob.Response(json={'message': 'OK'}, status=202)
 
     def version_discovery(self):
         return webob.Response(json={'api_version': api_server.VERSION})
+
+    def set_interface_rules(self, ip_addr):
+        interface_webob = self._amphora_info.get_interface(ip_addr)
+
+        if interface_webob.status_code != 200:
+            return interface_webob
+        interface = interface_webob.json['interface']
+
+        try:
+            rules_info = flask.request.get_json()
+            validate(rules_info, rules_schema.SUPPORTED_RULES_SCHEMA)
+        except Exception as e:
+            raise exceptions.BadRequest(
+                description='Invalid rules information') from e
+
+        nftable_utils.write_nftable_rules_file(interface, rules_info)
+
+        nftable_utils.load_nftables_file()
+
+        return webob.Response(json={'message': 'OK'}, status=200)
