@@ -37,10 +37,6 @@ from octavia import version
 
 LOG = logging.getLogger(__name__)
 
-EXTRA_LOG_LEVEL_DEFAULTS = [
-    'neutronclient.v2_0.client=INFO',
-]
-
 core_opts = [
     cfg.HostnameOpt('host', default=utils.get_hostname(),
                     sample_default='<server-hostname.example.com>',
@@ -89,7 +85,7 @@ api_opts = [
                        'octavia.api.drivers entrypoint.'),
                 default={'amphora': 'The Octavia Amphora driver.',
                          'octavia': 'Deprecated alias of the Octavia Amphora '
-                         'driver.',
+                                    'driver.',
                          }),
     cfg.StrOpt('default_provider_driver', default='amphora',
                help=_('Default provider driver.')),
@@ -152,14 +148,6 @@ amphora_agent_opts = [
     cfg.StrOpt('agent_server_network_dir',
                help=_("The directory where new network interfaces "
                       "are located")),
-    cfg.StrOpt('agent_server_network_file',
-               help=_("The file where the network interfaces are located. "
-                      "Specifying this will override any value set for "
-                      "agent_server_network_dir."),
-               deprecated_for_removal=True,
-               deprecated_reason=_('New amphora interface management '
-                                   'does not support single interface file.'),
-               deprecated_since='Xena'),
     cfg.IntOpt('agent_request_read_timeout', default=180,
                help=_("The time in seconds to allow a request from the "
                       "controller to run before terminating the socket.")),
@@ -321,17 +309,10 @@ health_manager_opts = [
                default=10,
                mutable=True,
                help=_('Sleep time between sending heartbeats.')),
-
-    # Used for updating health
-    cfg.StrOpt('health_update_driver', default='health_db',
-               help=_('Driver for updating amphora health system.'),
-               deprecated_for_removal=True,
-               deprecated_reason=_('This driver interface was removed.'),
-               deprecated_since='Victoria'),
 ]
 
 oslo_messaging_opts = [
-    cfg.StrOpt('topic'),
+    cfg.StrOpt('topic', help=_('Topic (i.e. Queue) Name')),
 ]
 
 haproxy_amphora_opts = [
@@ -400,8 +381,14 @@ haproxy_amphora_opts = [
     cfg.StrOpt('haproxy_cmd', default='/usr/sbin/haproxy',
                help=_("The full path to haproxy")),
     cfg.IntOpt('respawn_count', default=2,
+               deprecated_for_removal=True,
+               deprecated_reason='upstart support has been removed and this '
+                                 'option is no longer used.',
                help=_("The respawn count for haproxy's upstart script")),
     cfg.IntOpt('respawn_interval', default=2,
+               deprecated_for_removal=True,
+               deprecated_reason='upstart support has been removed and this '
+                                 'option is no longer used.',
                help=_("The respawn interval for haproxy's upstart script")),
     cfg.FloatOpt('rest_request_conn_timeout', default=10,
                  help=_("The time in seconds to wait for a REST API "
@@ -426,11 +413,6 @@ haproxy_amphora_opts = [
                help=_("The client certificate to talk to the agent")),
     cfg.StrOpt('server_ca', default='/etc/octavia/certs/server_ca.pem',
                help=_("The ca which signed the server certificates")),
-    cfg.BoolOpt('use_upstart', default=True,
-                deprecated_for_removal=True,
-                deprecated_reason='This is now automatically discovered '
-                                  ' and configured.',
-                help=_("If False, use sysvinit.")),
     cfg.IntOpt('api_db_commit_retry_attempts', default=15,
                help=_('The number of times the database action will be '
                       'attempted.')),
@@ -511,9 +493,6 @@ controller_worker_opts = [
                default='distributor_noop_driver',
                help=_('Name of the distributor driver to use')),
     cfg.ListOpt('statistics_drivers', default=['stats_db'],
-                deprecated_name='stats_update_driver',
-                deprecated_group='health_manager',
-                deprecated_since='Victoria',
                 help=_('List of drivers for updating amphora statistics.')),
     cfg.StrOpt('loadbalancer_topology',
                default=constants.TOPOLOGY_SINGLE,
@@ -523,6 +502,10 @@ controller_worker_opts = [
                       'SINGLE - One amphora per load balancer. '
                       'ACTIVE_STANDBY - Two amphora per load balancer.')),
     cfg.BoolOpt('user_data_config_drive', default=False,
+                deprecated_for_removal=True,
+                deprecated_reason=_('User_data nova option is not used and is '
+                                    ' too small to replace the config_drive.'),
+                deprecated_since='Antelope(2023.1)',
                 help=_('If True, build cloud-init user-data that is passed '
                        'to the config drive on Amphora boot instead of '
                        'personality files. If False, utilize personality '
@@ -531,7 +514,21 @@ controller_worker_opts = [
                help=_('Number of times an amphora delete should be retried.')),
     cfg.IntOpt('amphora_delete_retry_interval', default=5,
                help=_('Time, in seconds, between amphora delete retries.')),
-    cfg.BoolOpt('event_notifications', default=True),
+    cfg.BoolOpt('event_notifications', default=True,
+                help=_('Enable octavia event notifications. See '
+                       'oslo_messaging_notifications section for additional '
+                       'requirements.')),
+    # 2000 attempts is around 2h45 with the default settings
+    cfg.IntOpt('db_commit_retry_attempts', default=2000,
+               help=_('The number of times the database action will be '
+                      'attempted.')),
+    cfg.IntOpt('db_commit_retry_initial_delay', default=1,
+               help=_('The initial delay before a retry attempt.')),
+    cfg.IntOpt('db_commit_retry_backoff', default=1,
+               help=_('The time to backoff retry attempts.')),
+    cfg.IntOpt('db_commit_retry_max', default=5,
+               help=_('The maximum amount of time to wait between retry '
+                      'attempts.')),
 ]
 
 task_flow_opts = [
@@ -558,20 +555,31 @@ task_flow_opts = [
                choices=[('redis_taskflow_driver',
                          'Driver that will use Redis to store job states.'),
                         ('zookeeper_taskflow_driver',
-                         'Driver that will use Zookeeper to store job states.')
-                        ],
+                         'Driver that will use Zookeeper to store job '
+                         'states.'),
+                        ('etcd_taskflow_driver',
+                         'Driver that will user Etcd to store job states.')],
                help='Jobboard backend driver that will monitor job state.'),
     cfg.ListOpt('jobboard_backend_hosts', default=['127.0.0.1'],
                 help='Jobboard backend server host(s).'),
     cfg.PortOpt('jobboard_backend_port', default=6379,
                 help='Jobboard backend server port'),
-    cfg.StrOpt('jobboard_backend_password', default='', secret=True,
+    cfg.StrOpt('jobboard_backend_username',
+               help='Jobboard backend server user name'),
+    cfg.StrOpt('jobboard_backend_password', secret=True,
                help='Jobboard backend server password'),
     cfg.StrOpt('jobboard_backend_namespace', default='octavia_jobboard',
                help='Jobboard name that should be used to store taskflow '
                     'job id and claims for it.'),
+    cfg.IntOpt('jobboard_redis_backend_db',
+               default=0, min=0,
+               help='Database ID in redis server.'),
     cfg.StrOpt('jobboard_redis_sentinel', default=None,
                help='Sentinel name if it is used for Redis.'),
+    cfg.StrOpt('jobboard_redis_sentinel_username',
+               help='Redis Sentinel server user name'),
+    cfg.StrOpt('jobboard_redis_sentinel_password', secret=True,
+               help='Redis Sentinel server password'),
     cfg.DictOpt('jobboard_redis_backend_ssl_options',
                 help='Redis jobboard backend ssl configuration options.',
                 default={'ssl': False,
@@ -579,6 +587,13 @@ task_flow_opts = [
                          'ssl_certfile': None,
                          'ssl_ca_certs': None,
                          'ssl_cert_reqs': 'required'}),
+    cfg.DictOpt('jobboard_redis_sentinel_ssl_options',
+                help='Redis sentinel ssl configuration options.',
+                default={'ssl': False,
+                         'ssl_keyfile': None,
+                         'ssl_certfile': None,
+                         'ssl_ca_certs': None,
+                         'ssl_cert_reqs': 'required'}),
     cfg.DictOpt('jobboard_zookeeper_ssl_options',
                 help='Zookeeper jobboard backend ssl configuration options.',
                 default={'use_ssl': False,
@@ -586,6 +601,16 @@ task_flow_opts = [
                          'keyfile_password': None,
                          'certfile': None,
                          'verify_certs': True}),
+    cfg.DictOpt('jobboard_etcd_ssl_options',
+                help='Etcd jobboard backend ssl configuration options.',
+                default={'use_ssl': False,
+                         'ca_cert': None,
+                         'cert_key': None,
+                         'cert_cert': None}),
+    cfg.IntOpt('jobboard_etcd_timeout', default=None,
+               help='Timeout when communicating with the Etcd backend.'),
+    cfg.StrOpt('jobboard_etcd_api_path', default=None,
+               help='API Path of the Etcd server.'),
     cfg.IntOpt('jobboard_expiration_time', default=30,
                help='For backends like redis claiming jobs requiring setting '
                     'the expiry - how many seconds the claim should be '
@@ -740,21 +765,27 @@ cinder_opts = [
 ]
 
 neutron_opts = [
-    cfg.StrOpt('service_name',
-               help=_('The name of the neutron service in the '
-                      'keystone catalog')),
     cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
-                                  'in the keystone catalog.')),
-    cfg.StrOpt('region_name',
-               help=_('Region in Identity service catalog to use for '
-                      'communication with the OpenStack services.')),
-    cfg.StrOpt('endpoint_type', default='publicURL',
-               help=_('Endpoint interface in identity service to use')),
+                                  'in the keystone catalog.'),
+               deprecated_for_removal=True,
+               deprecated_reason=_('The endpoint_override option defined by '
+                                   'keystoneauth1 is the new name for this '
+                                   'option.'),
+               deprecated_since='2023.2/Bobcat'),
+    cfg.StrOpt('endpoint_type', help=_('Endpoint interface in identity '
+                                       'service to use'),
+               deprecated_for_removal=True,
+               deprecated_reason=_('This option was replaced by the '
+                                   'valid_interfaces option defined by '
+                                   'keystoneauth.'),
+               deprecated_since='2023.2/Bobcat'),
     cfg.StrOpt('ca_certificates_file',
-               help=_('CA certificates file path')),
-    cfg.BoolOpt('insecure',
-                default=False,
-                help=_('Disable certificate validation on SSL connections ')),
+               help=_('CA certificates file path'),
+               deprecated_for_removal=True,
+               deprecated_reason=_('The cafile option defined by '
+                                   'keystoneauth1 is the new name for this '
+                                   'option.'),
+               deprecated_since='2023.2/Bobcat'),
 ]
 
 glance_opts = [
@@ -895,8 +926,16 @@ _SQL_CONNECTION_DEFAULT = 'sqlite://'
 db_options.set_defaults(cfg.CONF, connection=_SQL_CONNECTION_DEFAULT,
                         max_pool_size=10, max_overflow=20, pool_timeout=10)
 
-ks_loading.register_auth_conf_options(cfg.CONF, constants.SERVICE_AUTH)
-ks_loading.register_session_conf_options(cfg.CONF, constants.SERVICE_AUTH)
+
+def register_ks_options(group):
+    ks_loading.register_auth_conf_options(cfg.CONF, group)
+    ks_loading.register_session_conf_options(cfg.CONF, group)
+    ks_loading.register_adapter_conf_options(cfg.CONF, group,
+                                             include_deprecated=False)
+
+
+register_ks_options(constants.SERVICE_AUTH)
+register_ks_options('neutron')
 
 
 def register_cli_opts():
@@ -904,23 +943,59 @@ def register_cli_opts():
     logging.register_options(cfg.CONF)
 
 
+def handle_neutron_deprecations():
+    # Apply neutron deprecated options to their new setting if needed
+
+    # Basically: if the new option is not set and the value of the deprecated
+    # option is not the default, it means that the deprecated setting is still
+    # used in the config file:
+    # * convert it to a valid "new" value if needed
+    # * set it as the default for the new option
+    # Thus [neutron].<new_option> has an higher precedence than
+    # [neutron].<deprecated_option>
+    loc = cfg.CONF.get_location('endpoint', 'neutron')
+    new_loc = cfg.CONF.get_location('endpoint_override', 'neutron')
+    if not new_loc and loc and loc.location != cfg.Locations.opt_default:
+        cfg.CONF.set_default('endpoint_override', cfg.CONF.neutron.endpoint,
+                             'neutron')
+
+    loc = cfg.CONF.get_location('endpoint_type', 'neutron')
+    new_loc = cfg.CONF.get_location('valid_interfaces', 'neutron')
+    if not new_loc and loc and loc.location != cfg.Locations.opt_default:
+        endpoint_type = cfg.CONF.neutron.endpoint_type.replace('URL', '')
+        cfg.CONF.set_default('valid_interfaces', [endpoint_type],
+                             'neutron')
+
+    loc = cfg.CONF.get_location('ca_certificates_file', 'neutron')
+    new_loc = cfg.CONF.get_location('cafile', 'neutron')
+    if not new_loc and loc and loc.location != cfg.Locations.opt_default:
+        cfg.CONF.set_default('cafile', cfg.CONF.neutron.ca_certificates_file,
+                             'neutron')
+
+
 def init(args, **kwargs):
     register_cli_opts()
     cfg.CONF(args=args, project='octavia',
-             version='%%prog %s' % version.version_info.release_string(),
+             version=f'%prog {version.version_info.release_string()}',
              **kwargs)
     validate.check_default_tls_versions_min_conflict()
     setup_remote_debugger()
     validate.check_default_ciphers_prohibit_list_conflict()
 
+    # Override default auth_type for plugins with the default from service_auth
+    auth_type = cfg.CONF.service_auth.auth_type
+    cfg.CONF.set_default('auth_type', auth_type, 'neutron')
+
+    handle_neutron_deprecations()
+
 
 def setup_logging(conf):
     """Sets up the logging options for a log with supplied name.
 
     :param conf: a cfg.ConfOpts object
     """
-    logging.set_defaults(default_log_levels=logging.get_default_log_levels() +
-                         EXTRA_LOG_LEVEL_DEFAULTS)
+    ll = logging.get_default_log_levels()
+    logging.set_defaults(default_log_levels=ll)
     product_name = "octavia"
     logging.setup(conf, product_name)
     LOG.info("Logging enabled!")
@@ -937,6 +1012,7 @@ def _enable_pydev(debugger_host, debugger_port):
         import pydevd  # pylint: disable=import-outside-toplevel
 
     pydevd.settrace(debugger_host,
+                    suspend=False,
                     port=int(debugger_port),
                     stdoutToServer=True,
                     stderrToServer=True)
