@@ -14,22 +14,18 @@
 
 import abc
 
-import six
 
-
-@six.add_metaclass(abc.ABCMeta)
-class ComputeBase(object):
+class ComputeBase(metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def build(self, name="amphora_name", amphora_flavor=None,
-              image_id=None, image_tag=None, image_owner=None,
-              key_name=None, sec_groups=None, network_ids=None,
-              config_drive_files=None, user_data=None, server_group_id=None):
+              image_tag=None, image_owner=None, key_name=None, sec_groups=None,
+              network_ids=None, config_drive_files=None, user_data=None,
+              server_group_id=None, availability_zone=None):
         """Build a new amphora.
 
         :param name: Optional name for Amphora
         :param amphora_flavor: Optionally specify a flavor
-        :param image_id: ID of the base image for the amphora instance
         :param image_tag: tag of the base image for the amphora instance
         :param key_name: Optionally specify a keypair
         :param sec_groups: Optionally specify list of security groups
@@ -46,6 +42,7 @@ class ComputeBase(object):
                           well or a string
         :param server_group_id: Optional server group id(uuid) which is used
                                 for anti_affinity feature
+        :param availability_zone: Name of the compute availability zone.
 
         :raises ComputeBuildException: if compute failed to build amphora
         :returns: UUID of amphora
@@ -67,10 +64,11 @@ class ComputeBase(object):
         """
 
     @abc.abstractmethod
-    def get_amphora(self, compute_id):
+    def get_amphora(self, compute_id, management_network_id=None):
         """Retrieve an amphora object
 
-        :param compute_id: the id of the desired amphora
+        :param compute_id: the compute id of the desired amphora
+        :param management_network_id: ID of the management network
         :returns: the amphora object
         :returns: fault message or None
         """
@@ -123,3 +121,13 @@ class ComputeBase(object):
         :raises: NotFound
         :raises: NotImplementedError
         """
+
+    @abc.abstractmethod
+    def validate_availability_zone(self, availability_zone):
+        """Validates that a compute availability zone exists.
+
+        :param availability_zone: Name of the compute availability zone.
+        :return: None
+        :raises: NotFound
+        :raises: NotImplementedError
+        """
