@@ -13,7 +13,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 #
-
 from oslo_config import cfg
 from oslo_log import log as logging
 from taskflow.patterns import linear_flow
@@ -32,22 +31,26 @@ from octavia.controller.worker.v2.tasks import database_tasks
 from octavia.controller.worker.v2.tasks import lifecycle_tasks
 from octavia.controller.worker.v2.tasks import network_tasks
 from octavia.controller.worker.v2.tasks import notification_tasks
+from octavia.db import api as db_apis
 from octavia.db import repositories as repo
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
-class LoadBalancerFlows(object):
+class LoadBalancerFlows:
 
     def __init__(self):
         self.amp_flows = amphora_flows.AmphoraFlows()
+        self.amphora_repo = repo.AmphoraRepository()
+        self.amphora_member_port_repo = repo.AmphoraMemberPortRepository()
         self.listener_flows = listener_flows.ListenerFlows()
         self.pool_flows = pool_flows.PoolFlows()
         self.member_flows = member_flows.MemberFlows()
         self.lb_repo = repo.LoadBalancerRepository()
 
-    def get_create_load_balancer_flow(self, topology, listeners=None):
+    def get_create_load_balancer_flow(self, topology, listeners=None,
+                                      flavor_dict=None):
         """Creates a conditional graph flow that allocates a loadbalancer.
 
         :raises InvalidTopology: Invalid topology specified
@@ -59,7 +62,7 @@ class LoadBalancerFlows(object):
         lb_create_flow.add(lifecycle_tasks.LoadBalancerIDToErrorOnRevertTask(
             requires=constants.LOADBALANCER_ID))
 
-        # allocate VIP
+        # allocate VIP - Saves the VIP IP(s) in neutron
         lb_create_flow.add(database_tasks.ReloadLoadBalancer(
             name=constants.RELOAD_LB_BEFOR_ALLOCATE_VIP,
             requires=constants.LOADBALANCER_ID,
@@ -81,9 +84,11 @@ class LoadBalancerFlows(object):
             provides=constants.SUBNET))
 
         if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
-            lb_create_flow.add(*self._create_active_standby_topology())
+            lb_create_flow.add(*self._create_active_standby_topology(
+                flavor_dict=flavor_dict))
         elif topology == constants.TOPOLOGY_SINGLE:
-            lb_create_flow.add(*self._create_single_topology())
+            lb_create_flow.add(*self._create_single_topology(
+                flavor_dict=flavor_dict))
         else:
             LOG.error("Unknown topology: %s.  Unable to build load balancer.",
                       topology)
@@ -91,10 +96,12 @@ class LoadBalancerFlows(object):
 
         post_amp_prefix = constants.POST_LB_AMP_ASSOCIATION_SUBFLOW
         lb_create_flow.add(
-            self.get_post_lb_amp_association_flow(post_amp_prefix, topology))
+            self.get_post_lb_amp_association_flow(post_amp_prefix, topology,
+                                                  flavor_dict=flavor_dict))
 
         if listeners:
-            lb_create_flow.add(*self._create_listeners_flow())
+            lb_create_flow.add(
+                *self._create_listeners_flow(flavor_dict=flavor_dict))
 
         lb_create_flow.add(
             database_tasks.MarkLBActiveInDB(
@@ -112,7 +119,7 @@ class LoadBalancerFlows(object):
 
         return lb_create_flow
 
-    def _create_single_topology(self):
+    def _create_single_topology(self, flavor_dict=None):
         sf_name = (constants.ROLE_STANDALONE + '-' +
                    constants.AMP_PLUG_NET_SUBFLOW)
         amp_for_lb_net_flow = linear_flow.Flow(sf_name)
@@ -120,11 +127,13 @@ class LoadBalancerFlows(object):
             prefix=constants.ROLE_STANDALONE,
             role=constants.ROLE_STANDALONE)
         amp_for_lb_net_flow.add(amp_for_lb_flow)
-        amp_for_lb_net_flow.add(*self._get_amp_net_subflow(sf_name))
+        amp_for_lb_net_flow.add(*self._get_amp_net_subflow(
+            sf_name, flavor_dict=flavor_dict))
         return amp_for_lb_net_flow
 
     def _create_active_standby_topology(
-            self, lf_name=constants.CREATE_LOADBALANCER_FLOW):
+            self, lf_name=constants.CREATE_LOADBALANCER_FLOW,
+            flavor_dict=None):
         # When we boot up amphora for an active/standby topology,
         # we should leverage the Nova anti-affinity capabilities
         # to place the amphora on different hosts, also we need to check
@@ -156,26 +165,45 @@ class LoadBalancerFlows(object):
         master_amp_sf = linear_flow.Flow(master_sf_name)
         master_amp_sf.add(self.amp_flows.get_amphora_for_lb_subflow(
             prefix=constants.ROLE_MASTER, role=constants.ROLE_MASTER))
-        master_amp_sf.add(*self._get_amp_net_subflow(master_sf_name))
+        master_amp_sf.add(*self._get_amp_net_subflow(master_sf_name,
+                                                     flavor_dict=flavor_dict))
 
         backup_sf_name = (constants.ROLE_BACKUP + '-' +
                           constants.AMP_PLUG_NET_SUBFLOW)
         backup_amp_sf = linear_flow.Flow(backup_sf_name)
         backup_amp_sf.add(self.amp_flows.get_amphora_for_lb_subflow(
             prefix=constants.ROLE_BACKUP, role=constants.ROLE_BACKUP))
-        backup_amp_sf.add(*self._get_amp_net_subflow(backup_sf_name))
+        backup_amp_sf.add(*self._get_amp_net_subflow(backup_sf_name,
+                                                     flavor_dict=flavor_dict))
 
         amps_flow.add(master_amp_sf, backup_amp_sf)
 
         return flows + [amps_flow]
 
-    def _get_amp_net_subflow(self, sf_name):
+    def _get_amp_net_subflow(self, sf_name, flavor_dict=None):
         flows = []
-        flows.append(network_tasks.PlugVIPAmphora(
-            name=sf_name + '-' + constants.PLUG_VIP_AMPHORA,
-            requires=(constants.LOADBALANCER, constants.AMPHORA,
-                      constants.SUBNET),
-            provides=constants.AMP_DATA))
+        # If we have an SRIOV VIP, we need to setup a firewall in the amp
+        if flavor_dict and flavor_dict.get(constants.SRIOV_VIP, False):
+            flows.append(network_tasks.CreateSRIOVBasePort(
+                name=sf_name + '-' + constants.PLUG_VIP_AMPHORA,
+                requires=(constants.LOADBALANCER, constants.AMPHORA,
+                          constants.SUBNET),
+                provides=constants.PORT_DATA))
+            flows.append(compute_tasks.AttachPort(
+                name=sf_name + '-' + constants.ATTACH_PORT,
+                requires=(constants.AMPHORA),
+                rebind={constants.PORT: constants.PORT_DATA}))
+            flows.append(network_tasks.BuildAMPData(
+                name=sf_name + '-' + constants.BUILD_AMP_DATA,
+                requires=(constants.LOADBALANCER, constants.AMPHORA,
+                          constants.PORT_DATA),
+                provides=constants.AMP_DATA))
+        else:
+            flows.append(network_tasks.PlugVIPAmphora(
+                name=sf_name + '-' + constants.PLUG_VIP_AMPHORA,
+                requires=(constants.LOADBALANCER, constants.AMPHORA,
+                          constants.SUBNET),
+                provides=constants.AMP_DATA))
 
         flows.append(network_tasks.ApplyQosAmphora(
             name=sf_name + '-' + constants.APPLY_QOS_AMP,
@@ -196,7 +224,7 @@ class LoadBalancerFlows(object):
                       constants.AMPHORAE_NETWORK_CONFIG)))
         return flows
 
-    def _create_listeners_flow(self):
+    def _create_listeners_flow(self, flavor_dict=None):
         flows = []
         flows.append(
             database_tasks.ReloadLoadBalancer(
@@ -229,11 +257,13 @@ class LoadBalancerFlows(object):
             )
         )
         flows.append(
-            self.listener_flows.get_create_all_listeners_flow()
+            self.listener_flows.get_create_all_listeners_flow(
+                flavor_dict=flavor_dict)
         )
         return flows
 
-    def get_post_lb_amp_association_flow(self, prefix, topology):
+    def get_post_lb_amp_association_flow(self, prefix, topology,
+                                         flavor_dict=None):
         """Reload the loadbalancer and create networking subflows for
 
         created/allocated amphorae.
@@ -251,14 +281,15 @@ class LoadBalancerFlows(object):
             post_create_LB_flow.add(database_tasks.GetAmphoraeFromLoadbalancer(
                 requires=constants.LOADBALANCER_ID,
                 provides=constants.AMPHORAE))
-            vrrp_subflow = self.amp_flows.get_vrrp_subflow(prefix)
+            vrrp_subflow = self.amp_flows.get_vrrp_subflow(
+                prefix, flavor_dict=flavor_dict)
             post_create_LB_flow.add(vrrp_subflow)
 
         post_create_LB_flow.add(database_tasks.UpdateLoadbalancerInDB(
             requires=[constants.LOADBALANCER, constants.UPDATE_DICT]))
         return post_create_LB_flow
 
-    def _get_delete_listeners_flow(self, listeners):
+    def _get_delete_listeners_flow(self, listeners, flavor_dict=None):
         """Sets up an internal delete flow
 
         :param listeners: A list of listener dicts
@@ -268,7 +299,7 @@ class LoadBalancerFlows(object):
         for listener in listeners:
             listeners_delete_flow.add(
                 self.listener_flows.get_delete_listener_internal_flow(
-                    listener))
+                    listener, flavor_dict=flavor_dict))
         return listeners_delete_flow
 
     def get_delete_load_balancer_flow(self, lb):
@@ -308,6 +339,9 @@ class LoadBalancerFlows(object):
             pools_delete = self._get_delete_pools_flow(pools)
             delete_LB_flow.add(pools_delete)
             delete_LB_flow.add(listeners_delete)
+            member_ports_delete = self.get_delete_member_ports_subflow(
+                lb[constants.LOADBALANCER_ID])
+            delete_LB_flow.add(member_ports_delete)
         delete_LB_flow.add(network_tasks.UnplugVIP(
             requires=constants.LOADBALANCER))
         delete_LB_flow.add(network_tasks.DeallocateVIP(
@@ -344,6 +378,11 @@ class LoadBalancerFlows(object):
         update_LB_flow = linear_flow.Flow(constants.UPDATE_LOADBALANCER_FLOW)
         update_LB_flow.add(lifecycle_tasks.LoadBalancerToErrorOnRevertTask(
             requires=constants.LOADBALANCER))
+        update_LB_flow.add(network_tasks.UpdateVIPSecurityGroup(
+            requires=constants.LOADBALANCER_ID,
+            provides=constants.VIP_SG_ID))
+        update_LB_flow.add(network_tasks.UpdateAmphoraSecurityGroup(
+            requires=constants.LOADBALANCER_ID))
         update_LB_flow.add(network_tasks.ApplyQos(
             requires=(constants.LOADBALANCER, constants.UPDATE_DICT)))
         update_LB_flow.add(amphora_driver_tasks.ListenersUpdate(
@@ -466,12 +505,13 @@ class LoadBalancerFlows(object):
                     role=new_amp_role,
                     failed_amp_vrrp_port_id=failed_amp.get(
                         constants.VRRP_PORT_ID),
-                    is_vrrp_ipv6=failed_vrrp_is_ipv6))
+                    is_vrrp_ipv6=failed_vrrp_is_ipv6,
+                    flavor_dict=lb[constants.FLAVOR]))
         else:
             failover_LB_flow.add(
                 self.amp_flows.get_amphora_for_lb_failover_subflow(
                     prefix=constants.FAILOVER_LOADBALANCER_FLOW,
-                    role=new_amp_role))
+                    role=new_amp_role, flavor_dict=lb[constants.FLAVOR]))
 
         if lb_topology == constants.TOPOLOGY_ACTIVE_STANDBY:
             failover_LB_flow.add(database_tasks.MarkAmphoraBackupInDB(
@@ -593,7 +633,8 @@ class LoadBalancerFlows(object):
                 self.amp_flows.get_amphora_for_lb_failover_subflow(
                     prefix=(new_amp_role + '-' +
                             constants.FAILOVER_LOADBALANCER_FLOW),
-                    role=new_amp_role))
+                    role=new_amp_role,
+                    flavor_dict=lb[constants.FLAVOR]))
 
             failover_LB_flow.add(database_tasks.MarkAmphoraMasterInDB(
                 name=constants.MARK_AMP_MASTER_INDB,
@@ -680,7 +721,7 @@ class LoadBalancerFlows(object):
             failover_LB_flow.add(self.amp_flows.get_vrrp_subflow(
                 new_amp_role + '-' + constants.GET_VRRP_SUBFLOW,
                 timeout_dict, create_vrrp_group=False,
-                get_amphorae_status=False))
+                get_amphorae_status=False, flavor_dict=lb[constants.FLAVOR]))
 
             # #### End of standby ####
 
@@ -714,3 +755,30 @@ class LoadBalancerFlows(object):
                                             requires=constants.LOADBALANCER))
 
         return failover_LB_flow
+
+    def get_delete_member_ports_subflow(self, load_balancer_id):
+        """A subflow that will delete all of the member ports on an LB
+
+        :param load_balancer_id: A load balancer ID
+        :returns: A Taskflow flow
+        """
+        port_delete_flow = unordered_flow.Flow('delete_member_ports')
+
+        session = db_apis.get_session()
+        with session.begin():
+            amps = self.amphora_repo.get_amphorae_ids_on_lb(session,
+                                                            load_balancer_id)
+        for amp in amps:
+            with session.begin():
+                ports = self.amphora_member_port_repo.get_port_ids(session,
+                                                                   amp)
+                for port in ports:
+                    port_delete_flow.add(
+                        network_tasks.DeletePort(
+                            name='delete_member_port' + '-' + port,
+                            inject={constants.PORT_ID: port}))
+                    port_delete_flow.add(
+                        database_tasks.DeleteAmpMemberPortInDB(
+                            name='delete_member_port_in_db' + '-' + port,
+                            inject={constants.PORT_ID: port}))
+        return port_delete_flow
