@@ -13,6 +13,10 @@
 # under the License.
 #
 
+import copy
+from typing import List
+from typing import Optional
+
 from cryptography import fernet
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -57,7 +61,7 @@ class AmpRetry(retry.Times):
         max_retry_attempt = CONF.haproxy_amphora.connection_max_retries
         for task_name, ex_info in last_errors.items():
             if len(history) <= max_retry_attempt:
-                # When taskflow persistance is enabled and flow/task state is
+                # When taskflow persistence is enabled and flow/task state is
                 # saved in the backend. If flow(task) is restored(restart of
                 # worker,etc) we are getting ex_info as None - we need to RETRY
                 # task to check its real state.
@@ -79,47 +83,65 @@ class AmpListenersUpdate(BaseAmphoraTask):
         # health manager fix it.
         # TODO(johnsom) Optimize this to use the dicts and not need the
         #               DB lookups
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora[constants.ID])
         try:
-            db_lb = self.loadbalancer_repo.get(
-                db_apis.get_session(),
-                id=loadbalancer[constants.LOADBALANCER_ID])
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(session,
+                                               id=amphora[constants.ID])
+                db_lb = self.loadbalancer_repo.get(
+                    session,
+                    id=loadbalancer[constants.LOADBALANCER_ID])
             self.amphora_driver.update_amphora_listeners(
                 db_lb, db_amp, timeout_dict)
         except Exception as e:
             LOG.error('Failed to update listeners on amphora %s. Skipping '
                       'this amphora as it is failing to update due to: %s',
                       db_amp.id, str(e))
-            self.amphora_repo.update(db_apis.get_session(), db_amp.id,
-                                     status=constants.ERROR)
+            with session.begin():
+                self.amphora_repo.update(session, db_amp.id,
+                                         status=constants.ERROR)
 
 
 class AmphoraIndexListenerUpdate(BaseAmphoraTask):
     """Task to update the listeners on one amphora."""
 
-    def execute(self, loadbalancer, amphora_index, amphorae, timeout_dict=()):
+    def execute(self, loadbalancer, amphora_index, amphorae,
+                amphorae_status: dict, new_amphora_id: str, timeout_dict=()):
         # Note, we don't want this to cause a revert as it may be used
         # in a failover flow with both amps failing. Skip it and let
         # health manager fix it.
+
+        amphora_id = amphorae[amphora_index].get(constants.ID)
+        amphora_status = amphorae_status.get(amphora_id, {})
+        if amphora_status.get(constants.UNREACHABLE):
+            LOG.warning("Skipping listener update because amphora %s "
+                        "is not reachable.", amphora_id)
+            return
+
         try:
             # TODO(johnsom) Optimize this to use the dicts and not need the
             #               DB lookups
-            db_amp = self.amphora_repo.get(
-                db_apis.get_session(),
-                id=amphorae[amphora_index][constants.ID])
-            db_lb = self.loadbalancer_repo.get(
-                db_apis.get_session(),
-                id=loadbalancer[constants.LOADBALANCER_ID])
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(
+                    session,
+                    id=amphorae[amphora_index][constants.ID])
+                db_lb = self.loadbalancer_repo.get(
+                    session,
+                    id=loadbalancer[constants.LOADBALANCER_ID])
             self.amphora_driver.update_amphora_listeners(
                 db_lb, db_amp, timeout_dict)
         except Exception as e:
-            amphora_id = amphorae[amphora_index].get(constants.ID)
             LOG.error('Failed to update listeners on amphora %s. Skipping '
                       'this amphora as it is failing to update due to: %s',
                       amphora_id, str(e))
-            self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                     status=constants.ERROR)
+            # Update only the status of the newly created amphora during the
+            # failover
+            if amphora_id == new_amphora_id:
+                session = db_apis.get_session()
+                with session.begin():
+                    self.amphora_repo.update(session, amphora_id,
+                                             status=constants.ERROR)
 
 
 class ListenersUpdate(BaseAmphoraTask):
@@ -127,8 +149,10 @@ class ListenersUpdate(BaseAmphoraTask):
 
     def execute(self, loadbalancer_id):
         """Execute updates per listener for an amphora."""
-        loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                  id=loadbalancer_id)
+        session = db_apis.get_session()
+        with session.begin():
+            loadbalancer = self.loadbalancer_repo.get(session,
+                                                      id=loadbalancer_id)
         if loadbalancer:
             self.amphora_driver.update(loadbalancer)
         else:
@@ -139,8 +163,10 @@ class ListenersUpdate(BaseAmphoraTask):
         """Handle failed listeners updates."""
 
         LOG.warning("Reverting listeners updates.")
-        loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                  id=loadbalancer_id)
+        session = db_apis.get_session()
+        with session.begin():
+            loadbalancer = self.loadbalancer_repo.get(session,
+                                                      id=loadbalancer_id)
         for listener in loadbalancer.listeners:
             self.task_utils.mark_listener_prov_status_error(
                 listener.id)
@@ -151,12 +177,15 @@ class ListenersStart(BaseAmphoraTask):
 
     def execute(self, loadbalancer, amphora=None):
         """Execute listener start routines for listeners on an amphora."""
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         if db_lb.listeners:
             if amphora is not None:
-                db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                               id=amphora[constants.ID])
+                with session.begin():
+                    db_amp = self.amphora_repo.get(session,
+                                                   id=amphora[constants.ID])
             else:
                 db_amp = amphora
             self.amphora_driver.start(db_lb, db_amp)
@@ -166,8 +195,10 @@ class ListenersStart(BaseAmphoraTask):
         """Handle failed listeners starts."""
 
         LOG.warning("Reverting listeners starts.")
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         for listener in db_lb.listeners:
             self.task_utils.mark_listener_prov_status_error(listener.id)
 
@@ -176,27 +207,40 @@ class AmphoraIndexListenersReload(BaseAmphoraTask):
     """Task to reload all listeners on an amphora."""
 
     def execute(self, loadbalancer, amphora_index, amphorae,
-                timeout_dict=None):
+                amphorae_status: dict, new_amphora_id: str, timeout_dict=None):
         """Execute listener reload routines for listeners on an amphora."""
         if amphorae is None:
             return
+
+        amphora_id = amphorae[amphora_index].get(constants.ID)
+        amphora_status = amphorae_status.get(amphora_id, {})
+        if amphora_status.get(constants.UNREACHABLE):
+            LOG.warning("Skipping listener reload because amphora %s "
+                        "is not reachable.", amphora_id)
+            return
+
         # TODO(johnsom) Optimize this to use the dicts and not need the
         #               DB lookups
-        db_amp = self.amphora_repo.get(
-            db_apis.get_session(), id=amphorae[amphora_index][constants.ID])
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(),
-            id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(
+                session, id=amphorae[amphora_index][constants.ID])
+            db_lb = self.loadbalancer_repo.get(
+                session,
+                id=loadbalancer[constants.LOADBALANCER_ID])
         if db_lb.listeners:
             try:
                 self.amphora_driver.reload(db_lb, db_amp, timeout_dict)
             except Exception as e:
-                amphora_id = amphorae[amphora_index].id
                 LOG.warning('Failed to reload listeners on amphora %s. '
                             'Skipping this amphora as it is failing to '
                             'reload due to: %s', amphora_id, str(e))
-                self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                         status=constants.ERROR)
+                # Update only the status of the newly created amphora during
+                # the failover
+                if amphora_id == new_amphora_id:
+                    with session.begin():
+                        self.amphora_repo.update(session, amphora_id,
+                                                 status=constants.ERROR)
 
 
 class ListenerDelete(BaseAmphoraTask):
@@ -204,8 +248,10 @@ class ListenerDelete(BaseAmphoraTask):
 
     def execute(self, listener):
         """Execute listener delete routines for an amphora."""
-        db_listener = self.listener_repo.get(
-            db_apis.get_session(), id=listener[constants.LISTENER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_listener = self.listener_repo.get(
+                session, id=listener[constants.LISTENER_ID])
         self.amphora_driver.delete(db_listener)
         LOG.debug("Deleted the listener on the vip")
 
@@ -223,8 +269,10 @@ class AmphoraGetInfo(BaseAmphoraTask):
 
     def execute(self, amphora):
         """Execute get_info routine for an amphora."""
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora[constants.ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session,
+                                           id=amphora[constants.ID])
         self.amphora_driver.get_info(db_amp)
 
 
@@ -241,8 +289,10 @@ class AmphoraFinalize(BaseAmphoraTask):
 
     def execute(self, amphora):
         """Execute finalize_amphora routine."""
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora.get(constants.ID))
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session,
+                                           id=amphora.get(constants.ID))
         self.amphora_driver.finalize_amphora(db_amp)
         LOG.debug("Finalized the amphora.")
 
@@ -258,20 +308,31 @@ class AmphoraFinalize(BaseAmphoraTask):
 class AmphoraPostNetworkPlug(BaseAmphoraTask):
     """Task to notify the amphora post network plug."""
 
-    def execute(self, amphora, ports):
+    def execute(self, amphora, ports, amphora_network_config):
         """Execute post_network_plug routine."""
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora[constants.ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session,
+                                           id=amphora[constants.ID])
 
         for port in ports:
             net = data_models.Network(**port.pop(constants.NETWORK))
             ips = port.pop(constants.FIXED_IPS)
-            fixed_ips = [data_models.FixedIP(
-                subnet=data_models.Subnet(**ip.pop(constants.SUBNET)), **ip)
-                for ip in ips]
+            fixed_ips = []
+            for ip in ips:
+                subnet_arg = ip.pop(constants.SUBNET)
+                host_routes = subnet_arg.get('host_routes')
+                if host_routes:
+                    subnet_arg['host_routes'] = [
+                        data_models.HostRoute(**hr)
+                        for hr in host_routes
+                    ]
+                fixed_ips.append(data_models.FixedIP(
+                    subnet=data_models.Subnet(**subnet_arg), **ip))
             self.amphora_driver.post_network_plug(
-                db_amp, data_models.Port(network=net, fixed_ips=fixed_ips,
-                                         **port))
+                db_amp,
+                data_models.Port(network=net, fixed_ips=fixed_ips, **port),
+                amphora_network_config)
 
             LOG.debug("post_network_plug called on compute instance "
                       "%(compute_id)s for port %(port_id)s",
@@ -289,22 +350,27 @@ class AmphoraPostNetworkPlug(BaseAmphoraTask):
 class AmphoraePostNetworkPlug(BaseAmphoraTask):
     """Task to notify the amphorae post network plug."""
 
-    def execute(self, loadbalancer, added_ports):
+    def execute(self, loadbalancer, updated_ports, amphorae_network_config):
         """Execute post_network_plug routine."""
         amp_post_plug = AmphoraPostNetworkPlug()
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         for amphora in db_lb.amphorae:
-            if amphora.id in added_ports:
+            if amphora.id in updated_ports:
                 amp_post_plug.execute(amphora.to_dict(),
-                                      added_ports[amphora.id])
+                                      updated_ports[amphora.id],
+                                      amphorae_network_config[amphora.id])
 
-    def revert(self, result, loadbalancer, added_ports, *args, **kwargs):
+    def revert(self, result, loadbalancer, updated_ports, *args, **kwargs):
         """Handle a failed post network plug."""
         if isinstance(result, failure.Failure):
             return
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         LOG.warning("Reverting post network plug.")
         for amphora in filter(
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
@@ -318,23 +384,48 @@ class AmphoraPostVIPPlug(BaseAmphoraTask):
 
     def execute(self, amphora, loadbalancer, amphorae_network_config):
         """Execute post_vip_routine."""
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora.get(constants.ID))
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session,
+                                           id=amphora.get(constants.ID))
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         vrrp_port = data_models.Port(
             **amphorae_network_config[
                 amphora.get(constants.ID)][constants.VRRP_PORT])
         # Required for noop-case
-        vip_arg = amphorae_network_config[amphora.get(
-            constants.ID)][constants.VIP_SUBNET]
+        vip_arg = copy.deepcopy(
+            amphorae_network_config[
+                amphora.get(constants.ID)][constants.VIP_SUBNET])
         if vip_arg:
+            host_routes = vip_arg.get('host_routes')
+            if host_routes:
+                vip_arg['host_routes'] = [
+                    data_models.HostRoute(**hr)
+                    for hr in host_routes
+                ]
             vip_subnet = data_models.Subnet(**vip_arg)
         else:
             vip_subnet = data_models.Subnet()
+
+        additional_vip_data = []
+        for add_vip in amphorae_network_config[
+                amphora[constants.ID]]['additional_vip_data']:
+
+            subnet_arg = copy.deepcopy(add_vip['subnet'])
+            subnet_arg['host_routes'] = [
+                data_models.HostRoute(**hr)
+                for hr in subnet_arg['host_routes']]
+            subnet = data_models.Subnet(**subnet_arg)
+
+            additional_vip_data.append(
+                data_models.AdditionalVipData(
+                    ip_address=add_vip['ip_address'],
+                    subnet=subnet))
+
         self.amphora_driver.post_vip_plug(
-            db_amp, db_lb, amphorae_network_config, vrrp_port=vrrp_port,
-            vip_subnet=vip_subnet)
+            db_amp, db_lb, amphorae_network_config, vrrp_port,
+            vip_subnet, additional_vip_data=additional_vip_data)
         LOG.debug("Notified amphora of vip plug")
 
     def revert(self, result, amphora, loadbalancer, *args, **kwargs):
@@ -343,8 +434,6 @@ class AmphoraPostVIPPlug(BaseAmphoraTask):
             return
         LOG.warning("Reverting post vip plug.")
         self.task_utils.mark_amphora_status_error(amphora.get(constants.ID))
-        self.task_utils.mark_loadbalancer_prov_status_error(
-            loadbalancer[constants.LOADBALANCER_ID])
 
 
 class AmphoraePostVIPPlug(BaseAmphoraTask):
@@ -353,21 +442,15 @@ class AmphoraePostVIPPlug(BaseAmphoraTask):
     def execute(self, loadbalancer, amphorae_network_config):
         """Execute post_vip_plug across the amphorae."""
         amp_post_vip_plug = AmphoraPostVIPPlug()
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         for amphora in db_lb.amphorae:
             amp_post_vip_plug.execute(amphora.to_dict(),
                                       loadbalancer,
                                       amphorae_network_config)
 
-    def revert(self, result, loadbalancer, *args, **kwargs):
-        """Handle a failed amphora vip plug notification."""
-        if isinstance(result, failure.Failure):
-            return
-        LOG.warning("Reverting amphorae post vip plug.")
-        self.task_utils.mark_loadbalancer_prov_status_error(
-            loadbalancer[constants.LOADBALANCER_ID])
-
 
 class AmphoraCertUpload(BaseAmphoraTask):
     """Upload a certificate to the amphora."""
@@ -377,8 +460,10 @@ class AmphoraCertUpload(BaseAmphoraTask):
         LOG.debug("Upload cert in amphora REST driver")
         key = utils.get_compatible_server_certs_key_passphrase()
         fer = fernet.Fernet(key)
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora.get(constants.ID))
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session,
+                                           id=amphora.get(constants.ID))
         self.amphora_driver.upload_cert_amp(
             db_amp, fer.decrypt(server_pem.encode('utf-8')))
 
@@ -391,8 +476,10 @@ class AmphoraUpdateVRRPInterface(BaseAmphoraTask):
         try:
             # TODO(johnsom) Optimize this to use the dicts and not need the
             #               DB lookups
-            db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                           id=amphora[constants.ID])
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(session,
+                                               id=amphora[constants.ID])
             interface = self.amphora_driver.get_interface_from_ip(
                 db_amp, db_amp.vrrp_ip, timeout_dict=timeout_dict)
         except Exception as e:
@@ -400,26 +487,37 @@ class AmphoraUpdateVRRPInterface(BaseAmphoraTask):
             LOG.error('Failed to get amphora VRRP interface on amphora '
                       '%s. Skipping this amphora as it is failing due to: '
                       '%s', amphora.get(constants.ID), str(e))
-            self.amphora_repo.update(db_apis.get_session(),
-                                     amphora.get(constants.ID),
-                                     status=constants.ERROR)
+            with session.begin():
+                self.amphora_repo.update(session,
+                                         amphora.get(constants.ID),
+                                         status=constants.ERROR)
             return None
 
-        self.amphora_repo.update(db_apis.get_session(), amphora[constants.ID],
-                                 vrrp_interface=interface)
+        with session.begin():
+            self.amphora_repo.update(session, amphora[constants.ID],
+                                     vrrp_interface=interface)
         return interface
 
 
 class AmphoraIndexUpdateVRRPInterface(BaseAmphoraTask):
     """Task to get and update the VRRP interface device name from amphora."""
 
-    def execute(self, amphora_index, amphorae, timeout_dict=None):
+    def execute(self, amphora_index, amphorae, amphorae_status: dict,
+                new_amphora_id: str, timeout_dict=None):
         amphora_id = amphorae[amphora_index][constants.ID]
+        amphora_status = amphorae_status.get(amphora_id, {})
+        if amphora_status.get(constants.UNREACHABLE):
+            LOG.warning("Skipping VRRP interface update because amphora %s "
+                        "is not reachable.", amphora_id)
+            return None
+
         try:
             # TODO(johnsom) Optimize this to use the dicts and not need the
             #               DB lookups
-            db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                           id=amphora_id)
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(session,
+                                               id=amphora_id)
             interface = self.amphora_driver.get_interface_from_ip(
                 db_amp, db_amp.vrrp_ip, timeout_dict=timeout_dict)
         except Exception as e:
@@ -427,12 +525,17 @@ class AmphoraIndexUpdateVRRPInterface(BaseAmphoraTask):
             LOG.error('Failed to get amphora VRRP interface on amphora '
                       '%s. Skipping this amphora as it is failing due to: '
                       '%s', amphora_id, str(e))
-            self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                     status=constants.ERROR)
+            # Update only the status of the newly created amphora during the
+            # failover
+            if amphora_id == new_amphora_id:
+                with session.begin():
+                    self.amphora_repo.update(session, amphora_id,
+                                             status=constants.ERROR)
             return None
 
-        self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                 vrrp_interface=interface)
+        with session.begin():
+            self.amphora_repo.update(session, amphora_id,
+                                     vrrp_interface=interface)
         return interface
 
 
@@ -449,10 +552,12 @@ class AmphoraVRRPUpdate(BaseAmphoraTask):
         try:
             # TODO(johnsom) Optimize this to use the dicts and not need the
             #               DB lookups
-            db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                           id=amphora_id)
-            loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                      id=loadbalancer_id)
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(session,
+                                               id=amphora_id)
+                loadbalancer = self.loadbalancer_repo.get(session,
+                                                          id=loadbalancer_id)
             db_amp.vrrp_interface = amp_vrrp_int
             self.amphora_driver.update_vrrp_conf(
                 loadbalancer, amphorae_network_config, db_amp, timeout_dict)
@@ -460,8 +565,9 @@ class AmphoraVRRPUpdate(BaseAmphoraTask):
             LOG.error('Failed to update VRRP configuration amphora %s. '
                       'Skipping this amphora as it is failing to update due '
                       'to: %s', amphora_id, str(e))
-            self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                     status=constants.ERROR)
+            with session.begin():
+                self.amphora_repo.update(session, amphora_id,
+                                         status=constants.ERROR)
 
         LOG.debug("Uploaded VRRP configuration of amphora %s.", amphora_id)
 
@@ -470,19 +576,28 @@ class AmphoraIndexVRRPUpdate(BaseAmphoraTask):
     """Task to update the VRRP configuration of an amphora."""
 
     def execute(self, loadbalancer_id, amphorae_network_config, amphora_index,
-                amphorae, amp_vrrp_int, timeout_dict=None):
+                amphorae, amphorae_status: dict, amp_vrrp_int: Optional[str],
+                new_amphora_id: str, timeout_dict=None):
         """Execute update_vrrp_conf."""
         # Note, we don't want this to cause a revert as it may be used
         # in a failover flow with both amps failing. Skip it and let
         # health manager fix it.
         amphora_id = amphorae[amphora_index][constants.ID]
+        amphora_status = amphorae_status.get(amphora_id, {})
+        if amphora_status.get(constants.UNREACHABLE):
+            LOG.warning("Skipping VRRP configuration because amphora %s "
+                        "is not reachable.", amphora_id)
+            return
+
         try:
             # TODO(johnsom) Optimize this to use the dicts and not need the
             #               DB lookups
-            db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                           id=amphora_id)
-            loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                      id=loadbalancer_id)
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(session,
+                                               id=amphora_id)
+                loadbalancer = self.loadbalancer_repo.get(session,
+                                                          id=loadbalancer_id)
             db_amp.vrrp_interface = amp_vrrp_int
             self.amphora_driver.update_vrrp_conf(
                 loadbalancer, amphorae_network_config, db_amp, timeout_dict)
@@ -490,8 +605,12 @@ class AmphoraIndexVRRPUpdate(BaseAmphoraTask):
             LOG.error('Failed to update VRRP configuration amphora %s. '
                       'Skipping this amphora as it is failing to update due '
                       'to: %s', amphora_id, str(e))
-            self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                     status=constants.ERROR)
+            # Update only the status of the newly created amphora during the
+            # failover
+            if amphora_id == new_amphora_id:
+                with session.begin():
+                    self.amphora_repo.update(session, amphora_id,
+                                             status=constants.ERROR)
             return
         LOG.debug("Uploaded VRRP configuration of amphora %s.", amphora_id)
 
@@ -505,8 +624,10 @@ class AmphoraVRRPStart(BaseAmphoraTask):
     def execute(self, amphora, timeout_dict=None):
         # TODO(johnsom) Optimize this to use the dicts and not need the
         #               DB lookups
-        db_amp = self.amphora_repo.get(
-            db_apis.get_session(), id=amphora[constants.ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(
+                session, id=amphora[constants.ID])
         self.amphora_driver.start_vrrp_service(db_amp, timeout_dict)
         LOG.debug("Started VRRP on amphora %s.", amphora[constants.ID])
 
@@ -517,19 +638,32 @@ class AmphoraIndexVRRPStart(BaseAmphoraTask):
     This will reload keepalived if it is already running.
     """
 
-    def execute(self, amphora_index, amphorae, timeout_dict=None):
+    def execute(self, amphora_index, amphorae, amphorae_status: dict,
+                new_amphora_id: str, timeout_dict=None):
         # TODO(johnsom) Optimize this to use the dicts and not need the
         #               DB lookups
         amphora_id = amphorae[amphora_index][constants.ID]
-        db_amp = self.amphora_repo.get(db_apis.get_session(), id=amphora_id)
+        amphora_status = amphorae_status.get(amphora_id, {})
+        if amphora_status.get(constants.UNREACHABLE):
+            LOG.warning("Skipping VRRP start because amphora %s "
+                        "is not reachable.", amphora_id)
+            return
+
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session, id=amphora_id)
         try:
             self.amphora_driver.start_vrrp_service(db_amp, timeout_dict)
         except Exception as e:
             LOG.error('Failed to start VRRP on amphora %s. '
                       'Skipping this amphora as it is failing to start due '
                       'to: %s', amphora_id, str(e))
-            self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                     status=constants.ERROR)
+            # Update only the status of the newly created amphora during the
+            # failover
+            if amphora_id == new_amphora_id:
+                with session.begin():
+                    self.amphora_repo.update(session, amphora_id,
+                                             status=constants.ERROR)
             return
         LOG.debug("Started VRRP on amphora %s.",
                   amphorae[amphora_index][constants.ID])
@@ -541,8 +675,10 @@ class AmphoraComputeConnectivityWait(BaseAmphoraTask):
     def execute(self, amphora, raise_retry_exception=False):
         """Execute get_info routine for an amphora until it responds."""
         try:
-            db_amphora = self.amphora_repo.get(
-                db_apis.get_session(), id=amphora.get(constants.ID))
+            session = db_apis.get_session()
+            with session.begin():
+                db_amphora = self.amphora_repo.get(
+                    session, id=amphora.get(constants.ID))
             amp_info = self.amphora_driver.get_info(
                 db_amphora, raise_retry_exception=raise_retry_exception)
             LOG.debug('Successfuly connected to amphora %s: %s',
@@ -552,9 +688,10 @@ class AmphoraComputeConnectivityWait(BaseAmphoraTask):
                       "This either means the compute driver failed to fully "
                       "boot the instance inside the timeout interval or the "
                       "instance is not reachable via the lb-mgmt-net.")
-            self.amphora_repo.update(db_apis.get_session(),
-                                     amphora.get(constants.ID),
-                                     status=constants.ERROR)
+            with session.begin():
+                self.amphora_repo.update(session,
+                                         amphora.get(constants.ID),
+                                         status=constants.ERROR)
             raise
 
 
@@ -573,8 +710,10 @@ class AmphoraConfigUpdate(BaseAmphoraTask):
         agent_cfg_tmpl = agent_jinja_cfg.AgentJinjaTemplater()
         agent_config = agent_cfg_tmpl.build_agent_config(
             amphora.get(constants.ID), topology)
-        db_amp = self.amphora_repo.get(db_apis.get_session(),
-                                       id=amphora[constants.ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session,
+                                           id=amphora[constants.ID])
         # Push the new configuration to the amphora
         try:
             self.amphora_driver.update_amphora_agent_config(db_amp,
@@ -584,3 +723,70 @@ class AmphoraConfigUpdate(BaseAmphoraTask):
                       'update. Please update the amphora image for this '
                       'amphora. Skipping.'.
                       format(amphora.get(constants.ID)))
+
+
+class AmphoraeGetConnectivityStatus(BaseAmphoraTask):
+    """Task that checks amphorae connectivity status.
+
+    Check and return the connectivity status of both amphorae in ACTIVE STANDBY
+    load balancers
+    """
+
+    def execute(self, amphorae: List[dict], new_amphora_id: str,
+                timeout_dict=None):
+        amphorae_status = {}
+
+        for amphora in amphorae:
+            amphora_id = amphora[constants.ID]
+            amphorae_status[amphora_id] = {}
+
+            session = db_apis.get_session()
+            with session.begin():
+                db_amp = self.amphora_repo.get(session, id=amphora_id)
+
+            try:
+                # Verify if the amphora is reachable
+                self.amphora_driver.check(db_amp, timeout_dict=timeout_dict)
+            except Exception as e:
+                LOG.exception("Cannot get status for amphora %s",
+                              amphora_id)
+                # In case it fails and the tested amphora is the newly created
+                # amphora, it's not a normal error handling, re-raise the
+                # exception
+                if amphora_id == new_amphora_id:
+                    raise e
+                amphorae_status[amphora_id][constants.UNREACHABLE] = True
+            else:
+                amphorae_status[amphora_id][constants.UNREACHABLE] = False
+
+        return amphorae_status
+
+
+class SetAmphoraFirewallRules(BaseAmphoraTask):
+    """Task to push updated firewall ruls to an amphora."""
+
+    def execute(self, amphorae: List[dict], amphora_index: int,
+                amphora_firewall_rules: List[dict], amphorae_status: dict,
+                timeout_dict=None):
+
+        if (amphora_firewall_rules and
+                amphora_firewall_rules[0].get('non-sriov-vip', False)):
+            # Not an SRIOV VIP, so skip setting firewall rules.
+            # This is already logged in GetAmphoraFirewallRules.
+            return
+
+        amphora_id = amphorae[amphora_index][constants.ID]
+        amphora_status = amphorae_status.get(amphora_id, {})
+        if amphora_status.get(constants.UNREACHABLE):
+            LOG.warning("Skipping firewall rules update because amphora %s "
+                        "is not reachable.", amphora_id)
+            return
+
+        session = db_apis.get_session()
+        with session.begin():
+            db_amp = self.amphora_repo.get(session, id=amphora_id)
+
+        self.amphora_driver.set_interface_rules(
+            db_amp,
+            amphorae[amphora_index][constants.VRRP_IP],
+            amphora_firewall_rules, timeout_dict=timeout_dict)
