@@ -19,6 +19,7 @@ from oslo_db.sqlalchemy import models
 import sqlalchemy as sa
 from sqlalchemy.ext import orderinglist
 from sqlalchemy import orm
+from sqlalchemy.orm import Mapped
 from sqlalchemy.orm import validates
 from sqlalchemy.sql import func
 from sqlalchemy_utils import ScalarListType
@@ -184,7 +185,7 @@ class ListenerStatistics(base_models.BASE):
             self.total_connections += other.total_connections
         else:
             raise TypeError(  # noqa: O342
-                "unsupported operand type(s) for +=: '{0}' and '{1}'".format(
+                "unsupported operand type(s) for +=: '{}' and '{}'".format(
                     type(self), type(other)))
 
         return self
@@ -228,6 +229,7 @@ class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         nullable=False)
     enabled = sa.Column(sa.Boolean(), nullable=False)
     pool = orm.relationship("Pool", back_populates="members")
+    vnic_type = sa.Column(sa.String(64), nullable=True)
 
     _tags = orm.relationship(
         'Tags',
@@ -245,7 +247,7 @@ class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
                 f"ip_address={self.ip_address!r}, "
                 f"protocol_port={self.protocol_port!r}, "
                 f"operating_status={self.operating_status!r}, "
-                f"weight={self.weight!r})")
+                f"weight={self.weight!r}, vnic_type={self.vnic_type!r})")
 
 
 class HealthMonitor(base_models.BASE, base_models.IdMixin,
@@ -353,7 +355,8 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         "SessionPersistence", uselist=False, cascade="delete",
         back_populates="pool")
     _default_listeners = orm.relationship("Listener", uselist=True,
-                                          back_populates="default_pool")
+                                          back_populates="default_pool",
+                                          cascade_backrefs=False)
     l7policies = orm.relationship("L7Policy", uselist=True,
                                   back_populates="redirect_pool")
     _tags = orm.relationship(
@@ -453,6 +456,7 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
         sa.ForeignKey("availability_zone.name",
                       name="fk_load_balancer_availability_zone_name"),
         nullable=True)
+    flavor: Mapped["Flavor"] = orm.relationship("Flavor")
 
     def __str__(self):
         return (f"LoadBalancer(id={self.id!r}, name={self.name!r}, "
@@ -503,6 +507,15 @@ class Vip(base_models.BASE):
     network_id = sa.Column(sa.String(36), nullable=True)
     qos_policy_id = sa.Column(sa.String(36), nullable=True)
     octavia_owned = sa.Column(sa.Boolean(), nullable=True)
+    vnic_type = sa.Column(sa.String(64), nullable=True)
+
+    sgs = orm.relationship(
+        "VipSecurityGroup", cascade="all,delete-orphan",
+        uselist=True, backref=orm.backref("vip", uselist=False))
+
+    @property
+    def sg_ids(self) -> list[str]:
+        return [sg.sg_id for sg in self.sgs]
 
 
 class AdditionalVip(base_models.BASE):
@@ -573,7 +586,8 @@ class Listener(base_models.BASE, base_models.IdMixin,
     load_balancer = orm.relationship("LoadBalancer", uselist=False,
                                      back_populates="listeners")
     default_pool = orm.relationship("Pool", uselist=False,
-                                    back_populates="_default_listeners")
+                                    back_populates="_default_listeners",
+                                    cascade_backrefs=False)
     sni_containers = orm.relationship(
         'SNI', cascade='all,delete-orphan',
         uselist=True, backref=orm.backref('listener', uselist=False))
@@ -599,6 +613,9 @@ class Listener(base_models.BASE, base_models.IdMixin,
     tls_ciphers = sa.Column(sa.String(2048), nullable=True)
     tls_versions = sa.Column(ScalarListType(), nullable=True)
     alpn_protocols = sa.Column(ScalarListType(), nullable=True)
+    hsts_max_age = sa.Column(sa.Integer, nullable=True)
+    hsts_include_subdomains = sa.Column(sa.Boolean, nullable=True)
+    hsts_preload = sa.Column(sa.Boolean, nullable=True)
 
     _tags = orm.relationship(
         'Tags',
@@ -907,6 +924,7 @@ class Flavor(base_models.BASE,
         sa.ForeignKey("flavor_profile.id",
                       name="fk_flavor_flavor_profile_id"),
         nullable=False)
+    flavor_profile: Mapped["FlavorProfile"] = orm.relationship("FlavorProfile")
 
 
 class AvailabilityZoneProfile(base_models.BASE, base_models.IdMixin,
@@ -942,6 +960,8 @@ class AvailabilityZone(base_models.BASE,
         sa.ForeignKey("availability_zone_profile.id",
                       name="fk_az_az_profile_id"),
         nullable=False)
+    availability_zone_profile: Mapped["AvailabilityZoneProfile"] = (
+        orm.relationship("AvailabilityZoneProfile"))
 
 
 class ClientAuthenticationMode(base_models.BASE):
@@ -965,3 +985,36 @@ class ListenerCidr(base_models.BASE):
         sa.ForeignKey("listener.id", name="fk_listener_cidr_listener_id"),
         nullable=False)
     cidr = sa.Column(sa.String(64), nullable=False)
+
+
+class VipSecurityGroup(base_models.BASE):
+
+    __data_model__ = data_models.VipSecurityGroup
+
+    __tablename__ = "vip_security_group"
+    __table_args__ = (
+        sa.PrimaryKeyConstraint('load_balancer_id', 'sg_id'),
+    )
+
+    load_balancer_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("vip.load_balancer_id", name="fk_vip_sg_vip_lb_id"),
+        nullable=False)
+    sg_id = sa.Column(sa.String(64), nullable=False)
+
+
+class AmphoraMemberPort(base_models.BASE, models.TimestampMixin):
+
+    __data_model__ = data_models.AmphoraMemberPort
+
+    __tablename__ = "amphora_member_port"
+
+    port_id = sa.Column(
+        sa.String(36),
+        primary_key=True)
+    amphora_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("amphora.id", name="fk_member_port_amphora_id"),
+        nullable=False)
+    network_id = sa.Column(
+        sa.String(36))
