@@ -111,8 +111,7 @@ class TestPlug(base.TestCase):
             )
         mock_webob.Response.assert_any_call(json={
             'message': 'OK',
-            'details': 'VIPs plugged on interface {interface}: {vips}'.format(
-                vips=FAKE_IP_IPV4, interface='eth1')
+            'details': f'VIPs plugged on interface eth1: {FAKE_IP_IPV4}'
         }, status=202)
 
     @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
@@ -140,8 +139,8 @@ class TestPlug(base.TestCase):
             )
         mock_webob.Response.assert_any_call(json={
             'message': 'OK',
-            'details': 'VIPs plugged on interface {interface}: {vips}'.format(
-                vips=FAKE_IP_IPV6_EXPANDED, interface='eth1')
+            'details': f'VIPs plugged on interface eth1: '
+                       f'{FAKE_IP_IPV6_EXPANDED}'
         }, status=202)
 
     @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
@@ -265,7 +264,10 @@ class TestPlug(base.TestCase):
                 "BaseOS.write_port_interface_file")
     @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
                 "BaseOS.bring_interface_up")
-    def test_plug_network(self, mock_if_up, mock_write_port_interface,
+    @mock.patch("octavia.amphorae.backends.agent.api_server.util."
+                "send_member_advertisements")
+    def test_plug_network(self, mock_send_member_adv,
+                          mock_if_up, mock_write_port_interface,
                           mock_netns, mock_iproute,
                           mock_by_mac, mock_interface_exists, mock_webob):
         fixed_ips = [
@@ -282,12 +284,13 @@ class TestPlug(base.TestCase):
             self.test_plug.plug_network(FAKE_MAC_ADDRESS, fixed_ips, 1400)
 
         mock_write_port_interface.assert_called_once_with(
-            interface='eth0', fixed_ips=fixed_ips, mtu=mtu)
-        mock_if_up.assert_called_once_with('eth0', 'network')
+            interface='eth2', fixed_ips=fixed_ips, mtu=mtu, is_sriov=False)
+        mock_if_up.assert_called_once_with('eth2', 'network')
+        mock_send_member_adv.assert_called_once_with(fixed_ips)
 
         mock_webob.Response.assert_any_call(
             json={'message': 'OK',
-                  'details': 'Plugged on interface eth0'},
+                  'details': 'Plugged on interface eth2'},
             status=202)
 
     @mock.patch.object(plug, "webob")
@@ -300,7 +303,10 @@ class TestPlug(base.TestCase):
                 "BaseOS.write_port_interface_file")
     @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
                 "BaseOS.bring_interface_up")
-    def test_plug_network_existing_interface(self, mock_if_up,
+    @mock.patch("octavia.amphorae.backends.agent.api_server.util."
+                "send_member_advertisements")
+    def test_plug_network_existing_interface(self, mock_send_member_adv,
+                                             mock_if_up,
                                              mock_write_port_interface,
                                              mock_netns, mock_by_mac,
                                              mock_interface_exists,
@@ -325,13 +331,14 @@ class TestPlug(base.TestCase):
             self.test_plug.plug_network(FAKE_MAC_ADDRESS, fixed_ips, 1400)
 
         mock_write_port_interface.assert_called_once_with(
-            interface=FAKE_INTERFACE, fixed_ips=fixed_ips, mtu=mtu)
+            interface=FAKE_INTERFACE, fixed_ips=fixed_ips, mtu=mtu,
+            is_sriov=False)
         mock_if_up.assert_called_once_with(FAKE_INTERFACE, 'network')
+        mock_send_member_adv.assert_called_once_with(fixed_ips)
 
         mock_webob.Response.assert_any_call(
             json={'message': 'OK',
-                  'details': 'Updated existing interface {}'.format(
-                      FAKE_INTERFACE)},
+                  'details': f'Updated existing interface {FAKE_INTERFACE}'},
             status=202)
 
     @mock.patch.object(plug, "webob")
@@ -344,8 +351,10 @@ class TestPlug(base.TestCase):
                 "BaseOS.write_vip_interface_file")
     @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
                 "BaseOS.bring_interface_up")
+    @mock.patch("octavia.amphorae.backends.agent.api_server.util."
+                "send_member_advertisements")
     def test_plug_network_on_vip(
-            self, mock_if_up, mock_write_vip_interface,
+            self, mock_send_member_adv, mock_if_up, mock_write_vip_interface,
             mock_netns, mock_by_mac, mock_interface_exists, mock_webob):
         fixed_ips = [
             {'ip_address': FAKE_IP_IPV4,
@@ -391,12 +400,49 @@ class TestPlug(base.TestCase):
                 'gateway': vip_net_info['gateway'],
                 'host_routes': [],
             },
-            fixed_ips=fixed_ips, mtu=mtu)
+            fixed_ips=fixed_ips, mtu=mtu, is_sriov=False)
 
         mock_if_up.assert_called_once_with(FAKE_INTERFACE, 'vip')
+        mock_send_member_adv.assert_called_once_with(fixed_ips)
 
         mock_webob.Response.assert_any_call(
             json={'message': 'OK',
-                  'details': 'Updated existing interface {}'.format(
-                      FAKE_INTERFACE)},
+                  'details': f'Updated existing interface {FAKE_INTERFACE}'},
             status=202)
+
+    @mock.patch('pyroute2.NetNS', create=True)
+    def test__netns_get_next_interface(self, mock_netns):
+        netns_handle = mock_netns.return_value.__enter__.return_value
+
+        netns_handle.get_links.return_value = [
+            {'attrs': [['IFLA_IFNAME', 'lo']]},
+        ]
+
+        ifname = self.test_plug._netns_get_next_interface()
+        self.assertEqual('eth2', ifname)
+
+        netns_handle.get_links.return_value = [
+            {'attrs': [['IFLA_IFNAME', 'lo']]},
+            {'attrs': [['IFLA_IFNAME', 'eth1']]},
+            {'attrs': [['IFLA_IFNAME', 'eth2']]},
+            {'attrs': [['IFLA_IFNAME', 'eth3']]},
+        ]
+
+        ifname = self.test_plug._netns_get_next_interface()
+        self.assertEqual('eth4', ifname)
+
+        netns_handle.get_links.return_value = [
+            {'attrs': [['IFLA_IFNAME', 'lo']]},
+            {'attrs': [['IFLA_IFNAME', 'eth1']]},
+            {'attrs': [['IFLA_IFNAME', 'eth3']]},
+        ]
+
+        ifname = self.test_plug._netns_get_next_interface()
+        self.assertEqual('eth2', ifname)
+
+        netns_handle.get_links.return_value = [
+            {'attrs': [['IFLA_IFNAME', f'eth{idx}']]}
+            for idx in range(2, 1000)]
+
+        ifname = self.test_plug._netns_get_next_interface()
+        self.assertEqual('eth1000', ifname)
