@@ -17,7 +17,6 @@ from unittest import mock
 
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
-from oslo_utils.secretutils import md5
 from oslo_utils import uuidutils
 import requests
 import requests_mock
@@ -70,7 +69,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
             'api_version': API_VERSION}
         self.driver.clients[
             API_VERSION].get_info.return_value = {
-            'haproxy_version': u'1.6.3-1ubuntu0.1',
+            'haproxy_version': '1.6.3-1ubuntu0.1',
             'api_version': API_VERSION}
         self.driver.jinja_combo = mock.MagicMock()
         self.driver.lvs_jinja = mock.MagicMock()
@@ -113,7 +112,8 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
                             'vrrp_ip': self.amp.vrrp_ip,
                             'mtu': FAKE_MTU,
                             'host_routes': host_routes_data,
-                            'additional_vips': []}
+                            'additional_vips': [],
+                            'is_sriov': False}
 
         self.timeout_dict = {constants.REQ_CONN_TIMEOUT: 1,
                              constants.REQ_READ_TIMEOUT: 2,
@@ -123,7 +123,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
             vip_subnet=network_models.Subnet(
                 id=self.lb.vip.subnet_id,
                 cidr=FAKE_VIP_SUBNET,
-                host_routes=[]))
+                host_routes=[])).to_dict(recurse=True)
 
     @mock.patch('octavia.amphorae.drivers.haproxy.rest_api_driver.'
                 'HaproxyAmphoraLoadBalancerDriver._process_secret')
@@ -167,7 +167,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.clients[API_VERSION].upload_config.assert_not_called()
         self.driver.clients[API_VERSION].reload_listener.assert_not_called()
 
-    @mock.patch('octavia.db.api.get_session')
+    @mock.patch('octavia.db.api.session')
     @mock.patch('octavia.db.repositories.ListenerRepository.update')
     @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
     def test_update_amphora_listeners_bad_cert(
@@ -176,12 +176,13 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_amphora.id = 'mock_amphora_id'
         mock_amphora.api_version = API_VERSION
 
-        mock_get_session.return_value = 'fake_session'
+        mock_session = mock_get_session().begin().__enter__()
+
         mock_load_cert.side_effect = [Exception]
         self.driver.update_amphora_listeners(self.lb,
                                              mock_amphora, self.timeout_dict)
         mock_list_update.assert_called_once_with(
-            'fake_session', self.lb.listeners[0].id,
+            mock_session, self.lb.listeners[0].id,
             provisioning_status=constants.ERROR,
             operating_status=constants.ERROR)
         self.driver.jinja_combo.build_config.assert_not_called()
@@ -352,9 +353,10 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_oslo.return_value = fake_context
         self.driver.cert_manager.get_secret.reset_mock()
         self.driver.cert_manager.get_secret.return_value = fake_secret
-        ref_md5 = md5(fake_secret, usedforsecurity=False).hexdigest()  # nosec
+        ref_md5 = hashlib.md5(
+            fake_secret, usedforsecurity=False).hexdigest()  # nosec
         ref_id = hashlib.sha1(fake_secret).hexdigest()  # nosec
-        ref_name = '{id}.pem'.format(id=ref_id)
+        ref_name = f'{ref_id}.pem'
 
         result = self.driver._process_secret(
             sample_listener, sample_listener.client_ca_tls_certificate_id,
@@ -416,11 +418,11 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         mock_load_certs.return_value = pool_data
         fake_pem = b'fake pem'
         mock_build_pem.return_value = fake_pem
-        ref_md5 = md5(fake_pem, usedforsecurity=False).hexdigest()  # nosec
-        ref_name = '{id}.pem'.format(id=pool_cert.id)
-        ref_path = '{cert_dir}/{lb_id}/{name}'.format(
-            cert_dir=fake_cert_dir, lb_id=sample_listener.load_balancer.id,
-            name=ref_name)
+        ref_md5 = hashlib.md5(
+            fake_pem, usedforsecurity=False).hexdigest()  # nosec
+        ref_name = f'{pool_cert.id}.pem'
+        ref_path = (f'{fake_cert_dir}/{sample_listener.load_balancer.id}/'
+                    f'{ref_name}')
         ref_ca_name = 'fake_ca.pem'
         ref_ca_path = '{cert_dir}/{lb_id}/{name}'.format(
             cert_dir=fake_cert_dir, lb_id=sample_listener.load_balancer.id,
@@ -635,7 +637,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
             # Ensure that at least one call in each pair has been seen
             if (dcp_calls[0] not in mock_calls and
                     dcp_calls[1] not in mock_calls):
-                raise Exception("%s not found in %s" % (dcp_calls, mock_calls))
+                raise Exception(f"{dcp_calls} not found in {mock_calls}")
 
         # Now just make sure we did an update and not a delete
         self.driver.clients[API_VERSION].delete_listener.assert_not_called()
@@ -698,43 +700,64 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.assertIsNone(result)
 
     def test_post_vip_plug(self):
-        amphorae_network_config = mock.MagicMock()
-        amphorae_network_config.get().vip_subnet.cidr = FAKE_CIDR
-        amphorae_network_config.get().vip_subnet.gateway_ip = FAKE_GATEWAY
-        amphorae_network_config.get().vip_subnet.host_routes = self.host_routes
-        amphorae_network_config.get().vip_subnet.to_dict.return_value = {
+        vip_subnet = mock.MagicMock()
+        vip_subnet.cidr = FAKE_CIDR
+        vip_subnet.gateway_ip = FAKE_GATEWAY
+        vip_subnet.host_routes = self.host_routes
+        vip_subnet.to_dict.return_value = {
             'cidr': FAKE_CIDR,
             'gateway_ip': FAKE_GATEWAY,
             'host_routes': [
                 hr.to_dict(recurse=True)
                 for hr in self.host_routes]
         }
+
+        amphorae_network_config = mock.MagicMock()
+        amphorae_network_config.get().vip_subnet = vip_subnet
         amphorae_network_config.get().vrrp_port = self.port
-        self.driver.post_vip_plug(self.amp, self.lb, amphorae_network_config)
+        self.driver.post_vip_plug(self.amp, self.lb, amphorae_network_config,
+                                  self.port, vip_subnet,
+                                  additional_vip_data=[])
         self.driver.clients[API_VERSION].plug_vip.assert_called_once_with(
             self.amp, self.lb.vip.ip_address, self.subnet_info)
 
     def test_post_vip_plug_additional_vips(self):
-        amphorae_network_config = mock.MagicMock()
-        amphorae_network_config.get().vip_subnet.cidr = FAKE_CIDR
-        amphorae_network_config.get().vip_subnet.gateway_ip = FAKE_GATEWAY
-        amphorae_network_config.get().vip_subnet.host_routes = self.host_routes
-        amphorae_network_config.get().vip_subnet.to_dict.return_value = {
+        vip_subnet = mock.MagicMock()
+        vip_subnet.cidr = FAKE_CIDR
+        vip_subnet.gateway_ip = FAKE_GATEWAY
+        vip_subnet.host_routes = self.host_routes
+        vip_subnet.to_dict.return_value = {
             'cidr': FAKE_CIDR,
             'gateway_ip': FAKE_GATEWAY,
             'host_routes': [
                 hr.to_dict(recurse=True)
                 for hr in self.host_routes]
         }
+
+        amphorae_network_config = mock.MagicMock()
+        amphorae_network_config.get().vip_subnet = vip_subnet
         amphorae_network_config.get().vrrp_port = self.port
+
+        vip1_subnet = mock.MagicMock()
+        vip1_subnet.cidr = mock.Mock()
+        vip1_subnet.gateway_ip = mock.Mock()
+        vip1_subnet.host_routes = self.host_routes
+        vip1_subnet.to_dict.return_value = {
+            'cidr': vip1_subnet.cidr,
+            'gateway_ip': vip1_subnet.gateway_ip,
+            'host_routes': [
+                hr.to_dict(recurse=True)
+                for hr in self.host_routes]
+        }
         additional_vip1 = mock.MagicMock()
         additional_vip1.ip_address = mock.Mock()
-        additional_vip1.subnet.cidr = mock.Mock()
-        additional_vip1.subnet.gateway_ip = mock.Mock()
-        additional_vip1.subnet.host_routes = self.host_routes
+        additional_vip1.subnet = vip1_subnet
         additional_vip_data = [additional_vip1]
-        self.driver.post_vip_plug(self.amp, self.lb, amphorae_network_config,
-                                  additional_vip_data=additional_vip_data)
+        self.driver.post_vip_plug(
+            self.amp, self.lb,
+            amphorae_network_config,
+            self.port, vip_subnet,
+            additional_vip_data=additional_vip_data)
         netinfo = self.subnet_info.copy()
         netinfo['additional_vips'] = [
             {
@@ -744,6 +767,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
                 'host_routes': netinfo['host_routes']
             }
         ]
+        netinfo['is_sriov'] = False
         self.driver.clients[API_VERSION].plug_vip.assert_called_once_with(
             self.amp, self.lb.vip.ip_address, netinfo)
 
@@ -756,7 +780,8 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.clients[API_VERSION].plug_network.assert_called_once_with(
             self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
                            fixed_ips=[],
-                           mtu=FAKE_MTU))
+                           mtu=FAKE_MTU,
+                           is_sriov=False))
 
         self.driver.clients[API_VERSION].plug_network.reset_mock()
 
@@ -768,7 +793,7 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
                                            subnet_cidr='198.51.100.0/24',
                                            host_routes=[],
                                            gateway=FAKE_GATEWAY)],
-                           mtu=FAKE_MTU))
+                           mtu=FAKE_MTU, is_sriov=False))
 
         self.driver.clients[API_VERSION].plug_network.reset_mock()
 
@@ -793,8 +818,9 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
                                vrrp_ip=self.amp.vrrp_ip,
                                host_routes=[],
                                additional_vips=[],
-                               mtu=FAKE_MTU
-                           )))
+                               mtu=FAKE_MTU,
+                               is_sriov=False
+                           ), is_sriov=False))
 
     def test_post_network_plug_with_host_routes(self):
         SUBNET_ID = 'SUBNET_ID'
@@ -834,7 +860,8 @@ class TestHaproxyAmphoraLoadBalancerDriverTest(base.TestCase):
         self.driver.clients[API_VERSION].plug_network.assert_called_once_with(
             self.amp, dict(mac_address=FAKE_MAC_ADDRESS,
                            fixed_ips=expected_fixed_ips,
-                           mtu=FAKE_MTU))
+                           mtu=FAKE_MTU,
+                           is_sriov=False))
 
     def test_get_haproxy_versions(self):
         ref_haproxy_versions = ['1', '6']
@@ -926,7 +953,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
     @requests_mock.mock()
     def test_get_api_version(self, mock_requests):
         ref_api_version = {'api_version': '0.1'}
-        mock_requests.get('{base}/'.format(base=self.base_url),
+        mock_requests.get(f'{self.base_url}/',
                           json=ref_api_version)
         result = self.driver.get_api_version(self.amp)
         self.assertEqual(ref_api_version, result)
@@ -934,8 +961,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     @requests_mock.mock()
     def test_get_info(self, m):
         info = {"hostname": "some_hostname", "version": "some_version",
-                "api_version": "0.5", "uuid": FAKE_UUID_1}
-        m.get("{base}/info".format(base=self.base_url_ver),
+                "api_version": "1.0", "uuid": FAKE_UUID_1}
+        m.get(f"{self.base_url_ver}/info",
               json=info)
         information = self.driver.get_info(self.amp)
         self.assertEqual(info, information)
@@ -943,8 +970,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     @requests_mock.mock()
     def test_get_info_with_timeout_dict(self, m):
         info = {"hostname": "some_hostname", "version": "some_version",
-                "api_version": "0.5", "uuid": FAKE_UUID_1}
-        m.get("{base}/info".format(base=self.base_url_ver),
+                "api_version": "1.0", "uuid": FAKE_UUID_1}
+        m.get(f"{self.base_url_ver}/info",
               json=info)
         timeout_dict = {
             constants.CONN_MAX_RETRIES: 100,
@@ -955,27 +982,27 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_get_info_unauthorized(self, m):
-        m.get("{base}/info".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/info",
               status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.get_info, self.amp)
 
     @requests_mock.mock()
     def test_get_info_missing(self, m):
-        m.get("{base}/info".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/info",
               status_code=404,
               headers={'content-type': 'application/json'})
         self.assertRaises(exc.NotFound, self.driver.get_info, self.amp)
 
     @requests_mock.mock()
     def test_get_info_server_error(self, m):
-        m.get("{base}/info".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/info",
               status_code=500)
         self.assertRaises(exc.InternalServerError, self.driver.get_info,
                           self.amp)
 
     @requests_mock.mock()
     def test_get_info_service_unavailable(self, m):
-        m.get("{base}/info".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/info",
               status_code=503)
         self.assertRaises(exc.ServiceUnavailable, self.driver.get_info,
                           self.amp)
@@ -983,37 +1010,37 @@ class TestAmphoraAPIClientTest(base.TestCase):
     @requests_mock.mock()
     def test_get_details(self, m):
         details = {"hostname": "some_hostname", "version": "some_version",
-                   "api_version": "0.5", "uuid": FAKE_UUID_1,
+                   "api_version": "1.0", "uuid": FAKE_UUID_1,
                    "network_tx": "some_tx", "network_rx": "some_rx",
                    "active": True, "haproxy_count": 10}
-        m.get("{base}/details".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/details",
               json=details)
         amp_details = self.driver.get_details(self.amp)
         self.assertEqual(details, amp_details)
 
     @requests_mock.mock()
     def test_get_details_unauthorized(self, m):
-        m.get("{base}/details".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/details",
               status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.get_details, self.amp)
 
     @requests_mock.mock()
     def test_get_details_missing(self, m):
-        m.get("{base}/details".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/details",
               status_code=404,
               headers={'content-type': 'application/json'})
         self.assertRaises(exc.NotFound, self.driver.get_details, self.amp)
 
     @requests_mock.mock()
     def test_get_details_server_error(self, m):
-        m.get("{base}/details".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/details",
               status_code=500)
         self.assertRaises(exc.InternalServerError, self.driver.get_details,
                           self.amp)
 
     @requests_mock.mock()
     def test_get_details_service_unavailable(self, m):
-        m.get("{base}/details".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/details",
               status_code=503)
         self.assertRaises(exc.ServiceUnavailable, self.driver.get_details,
                           self.amp)
@@ -1022,21 +1049,21 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_get_all_listeners(self, m):
         listeners = [{"status": "ONLINE", "provisioning_status": "ACTIVE",
                       "type": "PASSIVE", "uuid": FAKE_UUID_1}]
-        m.get("{base}/listeners".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/listeners",
               json=listeners)
         all_listeners = self.driver.get_all_listeners(self.amp)
         self.assertEqual(listeners, all_listeners)
 
     @requests_mock.mock()
     def test_get_all_listeners_unauthorized(self, m):
-        m.get("{base}/listeners".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/listeners",
               status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.get_all_listeners,
                           self.amp)
 
     @requests_mock.mock()
     def test_get_all_listeners_missing(self, m):
-        m.get("{base}/listeners".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/listeners",
               status_code=404,
               headers={'content-type': 'application/json'})
         self.assertRaises(exc.NotFound, self.driver.get_all_listeners,
@@ -1044,95 +1071,85 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_get_all_listeners_server_error(self, m):
-        m.get("{base}/listeners".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/listeners",
               status_code=500)
         self.assertRaises(exc.InternalServerError,
                           self.driver.get_all_listeners, self.amp)
 
     @requests_mock.mock()
     def test_get_all_listeners_service_unavailable(self, m):
-        m.get("{base}/listeners".format(base=self.base_url_ver),
+        m.get(f"{self.base_url_ver}/listeners",
               status_code=503)
         self.assertRaises(exc.ServiceUnavailable,
                           self.driver.get_all_listeners, self.amp)
 
     @requests_mock.mock()
     def test_start_loadbalancer(self, m):
-        m.put("{base}/loadbalancer/{loadbalancer_id}/start".format(
-            base=self.base_url_ver, loadbalancer_id=FAKE_UUID_1))
+        m.put(f"{self.base_url_ver}/loadbalancer/{FAKE_UUID_1}/start")
         self.driver.start_listener(self.amp, FAKE_UUID_1)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_start_loadbalancer_missing(self, m):
-        m.put("{base}/loadbalancer/{loadbalancer_id}/start".format(
-            base=self.base_url_ver, loadbalancer_id=FAKE_UUID_1),
-            status_code=404,
-            headers={'content-type': 'application/json'})
+        m.put(f"{self.base_url_ver}/loadbalancer/{FAKE_UUID_1}/start",
+              status_code=404,
+              headers={'content-type': 'application/json'})
         self.assertRaises(exc.NotFound, self.driver.start_listener,
                           self.amp, FAKE_UUID_1)
 
     @requests_mock.mock()
     def test_start_loadbalancer_unauthorized(self, m):
-        m.put("{base}/loadbalancer/{loadbalancer_id}/start".format(
-            base=self.base_url_ver, loadbalancer_id=FAKE_UUID_1),
-            status_code=401)
+        m.put(f"{self.base_url_ver}/loadbalancer/{FAKE_UUID_1}/start",
+              status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.start_listener,
                           self.amp, FAKE_UUID_1)
 
     @requests_mock.mock()
     def test_start_loadbalancer_server_error(self, m):
-        m.put("{base}/loadbalancer/{loadbalancer_id}/start".format(
-            base=self.base_url_ver, loadbalancer_id=FAKE_UUID_1),
-            status_code=500)
+        m.put(f"{self.base_url_ver}/loadbalancer/{FAKE_UUID_1}/start",
+              status_code=500)
         self.assertRaises(exc.InternalServerError, self.driver.start_listener,
                           self.amp, FAKE_UUID_1)
 
     @requests_mock.mock()
     def test_start_loadbalancer_service_unavailable(self, m):
-        m.put("{base}/loadbalancer/{loadbalancer_id}/start".format(
-            base=self.base_url_ver, loadbalancer_id=FAKE_UUID_1),
-            status_code=503)
+        m.put(f"{self.base_url_ver}/loadbalancer/{FAKE_UUID_1}/start",
+              status_code=503)
         self.assertRaises(exc.ServiceUnavailable, self.driver.start_listener,
                           self.amp, FAKE_UUID_1)
 
     @requests_mock.mock()
     def test_delete_listener(self, m):
-        m.delete("{base}/listeners/{listener_id}".format(
-            base=self.base_url_ver, listener_id=FAKE_UUID_1), json={})
+        m.delete(f"{self.base_url_ver}/listeners/{FAKE_UUID_1}", json={})
         self.driver.delete_listener(self.amp, FAKE_UUID_1)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_delete_listener_missing(self, m):
-        m.delete("{base}/listeners/{listener_id}".format(
-            base=self.base_url_ver, listener_id=FAKE_UUID_1),
-            status_code=404,
-            headers={'content-type': 'application/json'})
+        m.delete(f"{self.base_url_ver}/listeners/{FAKE_UUID_1}",
+                 status_code=404,
+                 headers={'content-type': 'application/json'})
         self.driver.delete_listener(self.amp, FAKE_UUID_1)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_delete_listener_unauthorized(self, m):
-        m.delete("{base}/listeners/{listener_id}".format(
-            base=self.base_url_ver, listener_id=FAKE_UUID_1),
-            status_code=401)
+        m.delete(f"{self.base_url_ver}/listeners/{FAKE_UUID_1}",
+                 status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.delete_listener,
                           self.amp, FAKE_UUID_1)
 
     @requests_mock.mock()
     def test_delete_listener_server_error(self, m):
-        m.delete("{base}/listeners/{listener_id}".format(
-            base=self.base_url_ver, listener_id=FAKE_UUID_1),
-            status_code=500)
+        m.delete(f"{self.base_url_ver}/listeners/{FAKE_UUID_1}",
+                 status_code=500)
         self.assertRaises(exc.InternalServerError, self.driver.delete_listener,
                           self.amp, FAKE_UUID_1)
 
     @requests_mock.mock()
     def test_delete_listener_service_unavailable(self, m):
-        m.delete("{base}/listeners/{listener_id}".format(
-            base=self.base_url_ver, listener_id=FAKE_UUID_1),
-            status_code=503)
+        m.delete(f"{self.base_url_ver}/listeners/{FAKE_UUID_1}",
+                 status_code=503)
         self.assertRaises(exc.ServiceUnavailable, self.driver.delete_listener,
                           self.amp, FAKE_UUID_1)
 
@@ -1189,14 +1206,14 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_update_cert_for_rotation(self, m):
-        m.put("{base}/certificate".format(base=self.base_url_ver))
+        m.put(f"{self.base_url_ver}/certificate")
         resp_body = self.driver.update_cert_for_rotation(self.amp,
                                                          "some_file")
         self.assertEqual(200, resp_body.status_code)
 
     @requests_mock.mock()
     def test_update_invalid_cert_for_rotation(self, m):
-        m.put("{base}/certificate".format(base=self.base_url_ver),
+        m.put(f"{self.base_url_ver}/certificate",
               status_code=400)
         self.assertRaises(exc.InvalidRequest,
                           self.driver.update_cert_for_rotation, self.amp,
@@ -1204,7 +1221,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_update_cert_for_rotation_unauthorized(self, m):
-        m.put("{base}/certificate".format(base=self.base_url_ver),
+        m.put(f"{self.base_url_ver}/certificate",
               status_code=401)
         self.assertRaises(exc.Unauthorized,
                           self.driver.update_cert_for_rotation, self.amp,
@@ -1212,7 +1229,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_update_cert_for_rotation_error(self, m):
-        m.put("{base}/certificate".format(base=self.base_url_ver),
+        m.put(f"{self.base_url_ver}/certificate",
               status_code=500)
         self.assertRaises(exc.InternalServerError,
                           self.driver.update_cert_for_rotation, self.amp,
@@ -1220,7 +1237,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_update_cert_for_rotation_unavailable(self, m):
-        m.put("{base}/certificate".format(base=self.base_url_ver),
+        m.put(f"{self.base_url_ver}/certificate",
               status_code=503)
         self.assertRaises(exc.ServiceUnavailable,
                           self.driver.update_cert_for_rotation, self.amp,
@@ -1331,10 +1348,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_config(self, m):
         config = {"name": "fake_config"}
         m.put(
-            "{base}/loadbalancer/{"
-            "amphora_id}/{loadbalancer_id}/haproxy".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                loadbalancer_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/loadbalancer/{self.amp.id}/"
+            f"{FAKE_UUID_1}/haproxy",
             json=config)
         self.driver.upload_config(self.amp, FAKE_UUID_1,
                                   config)
@@ -1344,10 +1359,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_invalid_config(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/loadbalancer/{"
-            "amphora_id}/{loadbalancer_id}/haproxy".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                loadbalancer_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/loadbalancer/{self.amp.id}/"
+            f"{FAKE_UUID_1}/haproxy",
             status_code=400)
         self.assertRaises(exc.InvalidRequest, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -1356,10 +1369,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_config_unauthorized(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/loadbalancer/{"
-            "amphora_id}/{loadbalancer_id}/haproxy".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                loadbalancer_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/loadbalancer/{self.amp.id}/"
+            f"{FAKE_UUID_1}/haproxy",
             status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -1368,10 +1379,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_config_server_error(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/loadbalancer/{"
-            "amphora_id}/{loadbalancer_id}/haproxy".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                loadbalancer_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/loadbalancer/{self.amp.id}/"
+            f"{FAKE_UUID_1}/haproxy",
             status_code=500)
         self.assertRaises(exc.InternalServerError, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -1380,10 +1389,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_config_service_unavailable(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/loadbalancer/{"
-            "amphora_id}/{loadbalancer_id}/haproxy".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                loadbalancer_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/loadbalancer/{self.amp.id}/"
+            f"{FAKE_UUID_1}/haproxy",
             status_code=503)
         self.assertRaises(exc.ServiceUnavailable, self.driver.upload_config,
                           self.amp, FAKE_UUID_1, config)
@@ -1392,10 +1399,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_udp_config(self, m):
         config = {"name": "fake_config"}
         m.put(
-            "{base}/listeners/"
-            "{amphora_id}/{listener_id}/udp_listener".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                listener_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/listeners/{self.amp.id}/"
+            f"{FAKE_UUID_1}/udp_listener",
             json=config)
         self.driver.upload_udp_config(self.amp, FAKE_UUID_1, config)
         self.assertTrue(m.called)
@@ -1404,10 +1409,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_udp_invalid_config(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/"
-            "{amphora_id}/{listener_id}/udp_listener".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                listener_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/listeners/{self.amp.id}/"
+            f"{FAKE_UUID_1}/udp_listener",
             status_code=400)
         self.assertRaises(exc.InvalidRequest, self.driver.upload_udp_config,
                           self.amp, FAKE_UUID_1, config)
@@ -1416,10 +1419,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_udp_config_unauthorized(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/"
-            "{amphora_id}/{listener_id}/udp_listener".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                listener_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/listeners/{self.amp.id}/"
+            f"{FAKE_UUID_1}/udp_listener",
             status_code=401)
         self.assertRaises(exc.Unauthorized, self.driver.upload_udp_config,
                           self.amp, FAKE_UUID_1, config)
@@ -1428,10 +1429,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_udp_config_server_error(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/"
-            "{amphora_id}/{listener_id}/udp_listener".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                listener_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/listeners/{self.amp.id}/"
+            f"{FAKE_UUID_1}/udp_listener",
             status_code=500)
         self.assertRaises(exc.InternalServerError,
                           self.driver.upload_udp_config,
@@ -1441,10 +1440,8 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_upload_udp_config_service_unavailable(self, m):
         config = '{"name": "bad_config"}'
         m.put(
-            "{base}/listeners/"
-            "{amphora_id}/{listener_id}/udp_listener".format(
-                amphora_id=self.amp.id, base=self.base_url_ver,
-                listener_id=FAKE_UUID_1),
+            f"{self.base_url_ver}/listeners/{self.amp.id}/"
+            f"{FAKE_UUID_1}/udp_listener",
             status_code=503)
         self.assertRaises(exc.ServiceUnavailable,
                           self.driver.upload_udp_config,
@@ -1452,18 +1449,14 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_plug_vip(self, m):
-        m.post("{base}/plug/vip/{vip}".format(
-            base=self.base_url_ver, vip=FAKE_IP)
-        )
+        m.post(f"{self.base_url_ver}/plug/vip/{FAKE_IP}")
         self.driver.plug_vip(self.amp, FAKE_IP, self.subnet_info)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_plug_vip_api_not_ready(self, m):
-        m.post("{base}/plug/vip/{vip}".format(
-            base=self.base_url_ver, vip=FAKE_IP),
-            status_code=404, headers={'content-type': 'text/html'}
-        )
+        m.post(f"{self.base_url_ver}/plug/vip/{FAKE_IP}",
+               status_code=404, headers={'content-type': 'text/html'})
         self.assertRaises(driver_except.TimeOutException,
                           self.driver.plug_vip,
                           self.amp, FAKE_IP, self.subnet_info)
@@ -1471,26 +1464,21 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_plug_network(self, m):
-        m.post("{base}/plug/network".format(
-            base=self.base_url_ver)
-        )
+        m.post(f"{self.base_url_ver}/plug/network")
         self.driver.plug_network(self.amp, self.port_info)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_upload_vrrp_config(self, m):
         config = '{"name": "bad_config"}'
-        m.put("{base}/vrrp/upload".format(
-            base=self.base_url_ver)
-        )
+        m.put(f"{self.base_url_ver}/vrrp/upload")
         self.driver.upload_vrrp_config(self.amp, config)
         self.assertTrue(m.called)
 
     @requests_mock.mock()
     def test_vrrp_action(self, m):
         action = 'start'
-        m.put("{base}/vrrp/{action}".format(base=self.base_url_ver,
-              action=action))
+        m.put(f"{self.base_url_ver}/vrrp/{action}")
         self.driver._vrrp_action(action, self.amp)
         self.assertTrue(m.called)
 
@@ -1498,8 +1486,7 @@ class TestAmphoraAPIClientTest(base.TestCase):
     def test_get_interface(self, m):
         interface = [{"interface": "eth1"}]
         ip_addr = '192.51.100.1'
-        m.get("{base}/interface/{ip_addr}".format(base=self.base_url_ver,
-                                                  ip_addr=ip_addr),
+        m.get(f"{self.base_url_ver}/interface/{ip_addr}",
               json=interface)
         self.driver.get_interface(self.amp, ip_addr)
         self.assertTrue(m.called)
@@ -1513,13 +1500,23 @@ class TestAmphoraAPIClientTest(base.TestCase):
 
     @requests_mock.mock()
     def test_update_agent_config(self, m):
-        m.put("{base}/config".format(base=self.base_url_ver))
+        m.put(f"{self.base_url_ver}/config")
         resp_body = self.driver.update_agent_config(self.amp, "some_file")
         self.assertEqual(200, resp_body.status_code)
 
     @requests_mock.mock()
     def test_update_agent_config_error(self, m):
-        m.put("{base}/config".format(base=self.base_url_ver), status_code=500)
+        m.put(f"{self.base_url_ver}/config", status_code=500)
         self.assertRaises(exc.InternalServerError,
                           self.driver.update_agent_config, self.amp,
                           "some_file")
+
+    @requests_mock.mock()
+    def test_set_interface_rules(self, m):
+        ip_addr = '192.0.2.44'
+        rules = ('[{"protocol":"TCP","cidr":"192.0.2.0/24","port":8080},'
+                 '{"protocol":"UDP","cidr":null,"port":80}]')
+        m.put(f'{self.base_url_ver}/interface/{ip_addr}/rules')
+
+        self.driver.set_interface_rules(self.amp, ip_addr, rules)
+        self.assertTrue(m.called)
