@@ -71,7 +71,7 @@ _session_mock = mock.MagicMock()
 @mock.patch('octavia.db.repositories.ListenerRepository.get',
             return_value=_listener_mock)
 @mock.patch('octavia.db.api.get_session', return_value=_session_mock)
-@mock.patch('octavia.controller.worker.v1.tasks.amphora_driver_tasks.LOG')
+@mock.patch('octavia.controller.worker.v2.tasks.amphora_driver_tasks.LOG')
 @mock.patch('oslo_utils.uuidutils.generate_uuid', return_value=AMP_ID)
 @mock.patch('stevedore.driver.DriverManager.driver')
 class TestAmphoraDriverTasks(base.TestCase):
@@ -132,23 +132,58 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_amphora_repo_get.return_value = _db_amphora_mock
         mock_lb_get.return_value = _db_load_balancer_mock
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: False
+            }
+        }
+
         amp_list_update_obj = amphora_driver_tasks.AmphoraIndexListenerUpdate()
         amp_list_update_obj.execute(_LB_mock, 0, [_amphora_mock],
+                                    amphorae_status,
+                                    _amphora_mock[constants.ID],
                                     self.timeout_dict)
 
         mock_driver.update_amphora_listeners.assert_called_once_with(
             _db_load_balancer_mock, _db_amphora_mock, self.timeout_dict)
 
+        # Unreachable amp
+        mock_driver.reset_mock()
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: True
+            }
+        }
+        amp_list_update_obj.execute(_LB_mock, 0, [_amphora_mock],
+                                    amphorae_status,
+                                    _amphora_mock[constants.ID],
+                                    self.timeout_dict)
+        mock_driver.update_amphora_listeners.assert_not_called()
+
+        # Test exception
         mock_driver.update_amphora_listeners.side_effect = Exception('boom')
 
-        amp_list_update_obj.execute(_LB_mock, 0,
-                                    [_amphora_mock], self.timeout_dict)
+        amp_list_update_obj.execute(_LB_mock, 0, [_amphora_mock], {},
+                                    _amphora_mock[constants.ID],
+                                    self.timeout_dict)
 
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, AMP_ID, status=constants.ERROR)
 
+        # Test exception, secondary amp
+        mock_amphora_repo_update.reset_mock()
+        mock_driver.update_amphora_listeners.side_effect = Exception('boom')
+
+        amp_list_update_obj.execute(_LB_mock, 0, [_amphora_mock], {},
+                                    '1234',
+                                    self.timeout_dict)
+
+        mock_amphora_repo_update.assert_not_called()
+
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
+    @mock.patch('octavia.db.api.session')
     def test_listeners_update(self,
+                              mock_get_session_ctx,
                               mock_lb_get,
                               mock_driver,
                               mock_generate_uuid,
@@ -174,15 +209,17 @@ class TestAmphoraDriverTasks(base.TestCase):
         listeners_update_obj.execute(None)
         mock_driver.update.assert_not_called()
 
+        mock_session = mock_get_session_ctx().begin().__enter__()
+
         # Test the revert
         amp = listeners_update_obj.revert(_LB_mock)
-        expected_db_calls = [mock.call(_session_mock,
+        expected_db_calls = [mock.call(mock_session,
                                        id=listeners[0].id,
                                        provisioning_status=constants.ERROR),
-                             mock.call(_session_mock,
+                             mock.call(mock_session,
                                        id=listeners[1].id,
                                        provisioning_status=constants.ERROR)]
-        repo.ListenerRepository.update.has_calls(expected_db_calls)
+        repo.ListenerRepository.update.assert_has_calls(expected_db_calls)
         self.assertEqual(2, repo.ListenerRepository.update.call_count)
         self.assertIsNone(amp)
 
@@ -194,39 +231,70 @@ class TestAmphoraDriverTasks(base.TestCase):
             mock_driver, mock_generate_uuid, mock_log, mock_get_session,
             mock_listener_repo_get, mock_listener_repo_update,
             mock_amphora_repo_get, mock_amphora_repo_update):
-        amphora_mock = mock.MagicMock()
         listeners_reload_obj = (
             amphora_driver_tasks.AmphoraIndexListenersReload())
         mock_lb = mock.MagicMock()
         mock_listener = mock.MagicMock()
         mock_listener.id = '12345'
-        mock_amphora_repo_get.return_value = amphora_mock
+        mock_amphora_repo_get.return_value = _amphora_mock
         mock_lb_repo_get.return_value = mock_lb
         mock_driver.reload.side_effect = [mock.DEFAULT, Exception('boom')]
 
         # Test no listeners
         mock_lb.listeners = None
-        listeners_reload_obj.execute(mock_lb, 0, None)
+        listeners_reload_obj.execute(mock_lb, 0, None, {},
+                                     _amphora_mock[constants.ID])
         mock_driver.reload.assert_not_called()
 
         # Test with listeners
-        mock_driver.start.reset_mock()
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: False
+            }
+        }
+        mock_driver.reload.reset_mock()
         mock_lb.listeners = [mock_listener]
-        listeners_reload_obj.execute(mock_lb, 0, [amphora_mock],
+        listeners_reload_obj.execute(mock_lb, 0, [_amphora_mock],
+                                     amphorae_status,
+                                     _amphora_mock[constants.ID],
                                      timeout_dict=self.timeout_dict)
-        mock_driver.reload.assert_called_once_with(mock_lb, amphora_mock,
+        mock_driver.reload.assert_called_once_with(mock_lb, _amphora_mock,
                                                    self.timeout_dict)
 
+        # Unreachable amp
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: True
+            }
+        }
+        mock_driver.reload.reset_mock()
+        listeners_reload_obj.execute(mock_lb, 0, [_amphora_mock],
+                                     amphorae_status,
+                                     _amphora_mock[constants.ID],
+                                     timeout_dict=self.timeout_dict)
+        mock_driver.reload.assert_not_called()
+
         # Test with reload exception
         mock_driver.reload.reset_mock()
-        listeners_reload_obj.execute(mock_lb, 0, [amphora_mock],
+        listeners_reload_obj.execute(mock_lb, 0, [_amphora_mock], {},
+                                     _amphora_mock[constants.ID],
                                      timeout_dict=self.timeout_dict)
-        mock_driver.reload.assert_called_once_with(mock_lb, amphora_mock,
+        mock_driver.reload.assert_called_once_with(mock_lb, _amphora_mock,
                                                    self.timeout_dict)
         mock_amphora_repo_update.assert_called_once_with(
-            _session_mock, amphora_mock[constants.ID],
+            _session_mock, _amphora_mock[constants.ID],
             status=constants.ERROR)
 
+        # Test with reload exception, secondary amp
+        mock_driver.reload.reset_mock()
+        mock_amphora_repo_update.reset_mock()
+        listeners_reload_obj.execute(mock_lb, 0, [_amphora_mock], {},
+                                     '1234',
+                                     timeout_dict=self.timeout_dict)
+        mock_driver.reload.assert_called_once_with(mock_lb, _amphora_mock,
+                                                   self.timeout_dict)
+        mock_amphora_repo_update.assert_not_called()
+
     @mock.patch('octavia.controller.worker.task_utils.TaskUtils.'
                 'mark_listener_prov_status_error')
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
@@ -262,7 +330,9 @@ class TestAmphoraDriverTasks(base.TestCase):
         listeners_start_obj.revert(_LB_mock)
         mock_prov_status_error.assert_called_once_with('12345')
 
+    @mock.patch('octavia.db.api.session')
     def test_listener_delete(self,
+                             mock_get_session_ctx,
                              mock_driver,
                              mock_generate_uuid,
                              mock_log,
@@ -278,10 +348,12 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.delete.assert_called_once_with(_listener_mock)
 
+        mock_session = mock_get_session_ctx().begin().__enter__()
+
         # Test the revert
         amp = listener_delete_obj.revert(listener_dict)
         repo.ListenerRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=LISTENER_ID,
             provisioning_status=constants.ERROR)
         self.assertIsNone(amp)
@@ -291,7 +363,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_listener_repo_update.side_effect = Exception('fail')
         amp = listener_delete_obj.revert(listener_dict)
         repo.ListenerRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=LISTENER_ID,
             provisioning_status=constants.ERROR)
         self.assertIsNone(amp)
@@ -330,7 +402,9 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_driver.get_diagnostics.assert_called_once_with(
             _amphora_mock)
 
+    @mock.patch('octavia.db.api.session')
     def test_amphora_finalize(self,
+                              mock_get_session_ctx,
                               mock_driver,
                               mock_generate_uuid,
                               mock_log,
@@ -347,10 +421,12 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_driver.finalize_amphora.assert_called_once_with(
             _db_amphora_mock)
 
+        mock_session = mock_get_session_ctx().begin().__enter__()
+
         # Test revert
         amp = amphora_finalize_obj.revert(None, _amphora_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
         self.assertIsNone(amp)
@@ -360,7 +436,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_amphora_repo_update.side_effect = Exception('fail')
         amp = amphora_finalize_obj.revert(None, _amphora_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
         self.assertIsNone(amp)
@@ -371,7 +447,9 @@ class TestAmphoraDriverTasks(base.TestCase):
             failure.Failure.from_exception(Exception('boom')), _amphora_mock)
         repo.AmphoraRepository.update.assert_not_called()
 
+    @mock.patch('octavia.db.api.session')
     def test_amphora_post_network_plug(self,
+                                       mock_get_session_ctx,
                                        mock_driver,
                                        mock_generate_uuid,
                                        mock_log,
@@ -396,10 +474,12 @@ class TestAmphoraDriverTasks(base.TestCase):
                                      network_data_models.Port(**port_mock),
                                      _amphora_network_config_mock)
 
+        mock_session = mock_get_session_ctx().begin().__enter__()
+
         # Test revert
         amp = amphora_post_network_plug_obj.revert(None, _amphora_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
 
@@ -410,7 +490,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_amphora_repo_update.side_effect = Exception('fail')
         amp = amphora_post_network_plug_obj.revert(None, _amphora_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
 
@@ -456,7 +536,10 @@ class TestAmphoraDriverTasks(base.TestCase):
             self.assertEqual(hr1['nexthop'], hr2.nexthop)
 
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
-    def test_amphorae_post_network_plug(self, mock_lb_get,
+    @mock.patch('octavia.db.api.session')
+    def test_amphorae_post_network_plug(self,
+                                        mock_get_session_ctx,
+                                        mock_lb_get,
                                         mock_driver,
                                         mock_generate_uuid,
                                         mock_log,
@@ -496,11 +579,13 @@ class TestAmphoraDriverTasks(base.TestCase):
                                               _amphora_network_config_mock)
         mock_driver.post_network_plug.assert_not_called()
 
+        mock_session = mock_get_session_ctx().begin().__enter__()
+
         # Test revert
         amp = amphora_post_network_plug_obj.revert(None, _LB_mock,
                                                    _deltas_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
 
@@ -512,7 +597,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         amp = amphora_post_network_plug_obj.revert(None, _LB_mock,
                                                    _deltas_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
 
@@ -527,7 +612,10 @@ class TestAmphoraDriverTasks(base.TestCase):
 
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
-    def test_amphora_post_vip_plug(self, mock_lb_get,
+    @mock.patch('octavia.db.api.session')
+    def test_amphora_post_vip_plug(self,
+                                   mock_get_session_ctx,
+                                   mock_lb_get,
                                    mock_loadbalancer_repo_update,
                                    mock_driver,
                                    mock_generate_uuid,
@@ -560,13 +648,14 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.post_vip_plug.assert_called_once_with(
             _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
-            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
-            additional_vip_data=[])
+            vrrp_port, vip_subnet, additional_vip_data=[])
+
+        mock_session = mock_get_session_ctx().begin().__enter__()
 
         # Test revert
         amp = amphora_post_vip_plug_obj.revert(None, _amphora_mock, _LB_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
         repo.LoadBalancerRepository.update.assert_not_called()
@@ -580,7 +669,7 @@ class TestAmphoraDriverTasks(base.TestCase):
         mock_loadbalancer_repo_update.side_effect = Exception('fail')
         amp = amphora_post_vip_plug_obj.revert(None, _amphora_mock, _LB_mock)
         repo.AmphoraRepository.update.assert_called_once_with(
-            _session_mock,
+            mock_session,
             id=AMP_ID,
             status=constants.ERROR)
         repo.LoadBalancerRepository.update.assert_not_called()
@@ -628,11 +717,10 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.post_vip_plug.assert_called_once_with(
             _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
-            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
-            additional_vip_data=[])
+            vrrp_port, vip_subnet, additional_vip_data=[])
 
-        call_kwargs = mock_driver.post_vip_plug.call_args[1]
-        vip_subnet_arg = call_kwargs.get(constants.VIP_SUBNET)
+        call_args = mock_driver.post_vip_plug.call_args[0]
+        vip_subnet_arg = call_args[4]
         self.assertEqual(2, len(vip_subnet_arg.host_routes))
         for hr1, hr2 in zip(host_routes, vip_subnet_arg.host_routes):
             self.assertEqual(hr1['destination'], hr2.destination)
@@ -691,11 +779,11 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.post_vip_plug.assert_called_once_with(
             _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
-            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
-            additional_vip_data=additional_vip_data)
+            vrrp_port, vip_subnet, additional_vip_data=additional_vip_data)
 
+        call_args = mock_driver.post_vip_plug.call_args[0]
         call_kwargs = mock_driver.post_vip_plug.call_args[1]
-        vip_subnet_arg = call_kwargs.get(constants.VIP_SUBNET)
+        vip_subnet_arg = call_args[4]
         self.assertEqual(2, len(vip_subnet_arg.host_routes))
         for hr1, hr2 in zip(host_routes, vip_subnet_arg.host_routes):
             self.assertEqual(hr1['destination'], hr2.destination)
@@ -741,8 +829,7 @@ class TestAmphoraDriverTasks(base.TestCase):
 
         mock_driver.post_vip_plug.assert_called_once_with(
             _db_amphora_mock, _db_load_balancer_mock, amphorae_net_config_mock,
-            vip_subnet=vip_subnet, vrrp_port=vrrp_port,
-            additional_vip_data=[])
+            vrrp_port, vip_subnet, additional_vip_data=[])
 
     def test_amphora_cert_upload(self,
                                  mock_driver,
@@ -804,6 +891,11 @@ class TestAmphoraDriverTasks(base.TestCase):
         FAKE_INTERFACE = 'fake0'
         mock_driver.get_interface_from_ip.side_effect = [FAKE_INTERFACE,
                                                          Exception('boom')]
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: False
+            }
+        }
 
         timeout_dict = {constants.CONN_MAX_RETRIES: CONN_MAX_RETRIES,
                         constants.CONN_RETRY_INTERVAL: CONN_RETRY_INTERVAL}
@@ -811,20 +903,39 @@ class TestAmphoraDriverTasks(base.TestCase):
         amphora_update_vrrp_interface_obj = (
             amphora_driver_tasks.AmphoraIndexUpdateVRRPInterface())
         amphora_update_vrrp_interface_obj.execute(
-            0, [_amphora_mock], timeout_dict)
+            0, [_amphora_mock], amphorae_status, _amphora_mock[constants.ID],
+            timeout_dict)
         mock_driver.get_interface_from_ip.assert_called_once_with(
             _db_amphora_mock, _db_amphora_mock.vrrp_ip,
             timeout_dict=timeout_dict)
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, _db_amphora_mock.id, vrrp_interface=FAKE_INTERFACE)
 
+        # Unreachable amp
+        mock_driver.reset_mock()
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: True
+            }
+        }
+        amphora_update_vrrp_interface_obj.execute(
+            0, [_amphora_mock], amphorae_status, _amphora_mock[constants.ID],
+            timeout_dict)
+        mock_driver.get_interface_from_ip.assert_not_called()
+
         # Test with an exception
         mock_amphora_repo_update.reset_mock()
         amphora_update_vrrp_interface_obj.execute(
-            0, [_amphora_mock], timeout_dict)
+            0, [_amphora_mock], {}, _amphora_mock[constants.ID], timeout_dict)
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, _db_amphora_mock.id, status=constants.ERROR)
 
+        # Test with an exception, secondary amp
+        mock_amphora_repo_update.reset_mock()
+        amphora_update_vrrp_interface_obj.execute(
+            0, [_amphora_mock], {}, '1234', timeout_dict)
+        mock_amphora_repo_update.assert_not_called()
+
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     def test_amphora_vrrp_update(self,
                                  mock_lb_get,
@@ -872,23 +983,52 @@ class TestAmphoraDriverTasks(base.TestCase):
                                                     Exception('boom')]
         mock_lb_get.return_value = _db_load_balancer_mock
         mock_amphora_repo_get.return_value = _db_amphora_mock
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: False
+            }
+        }
+
         amphora_vrrp_update_obj = (
             amphora_driver_tasks.AmphoraIndexVRRPUpdate())
 
         amphora_vrrp_update_obj.execute(LB_ID, amphorae_network_config,
-                                        0, [_amphora_mock], 'fakeint0',
+                                        0, [_amphora_mock], amphorae_status,
+                                        'fakeint0',
+                                        _amphora_mock[constants.ID],
                                         timeout_dict=self.timeout_dict)
         mock_driver.update_vrrp_conf.assert_called_once_with(
             _db_load_balancer_mock, amphorae_network_config, _db_amphora_mock,
             self.timeout_dict)
 
+        # Unreachable amp
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: True
+            }
+        }
+        mock_amphora_repo_update.reset_mock()
+        mock_driver.update_vrrp_conf.reset_mock()
+        amphora_vrrp_update_obj.execute(LB_ID, amphorae_network_config,
+                                        0, [_amphora_mock], amphorae_status,
+                                        None, _amphora_mock[constants.ID])
+        mock_driver.update_vrrp_conf.assert_not_called()
+
         # Test with an exception
         mock_amphora_repo_update.reset_mock()
         amphora_vrrp_update_obj.execute(LB_ID, amphorae_network_config,
-                                        0, [_amphora_mock], 'fakeint0')
+                                        0, [_amphora_mock], {}, 'fakeint0',
+                                        _amphora_mock[constants.ID])
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, _db_amphora_mock.id, status=constants.ERROR)
 
+        # Test with an exception, secondary amp
+        mock_amphora_repo_update.reset_mock()
+        amphora_vrrp_update_obj.execute(LB_ID, amphorae_network_config,
+                                        0, [_amphora_mock], {}, 'fakeint0',
+                                        '1234')
+        mock_amphora_repo_update.assert_not_called()
+
     def test_amphora_vrrp_start(self,
                                 mock_driver,
                                 mock_generate_uuid,
@@ -916,25 +1056,54 @@ class TestAmphoraDriverTasks(base.TestCase):
                                       mock_amphora_repo_get,
                                       mock_amphora_repo_update):
         mock_amphora_repo_get.return_value = _db_amphora_mock
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: False
+            }
+        }
+
         amphora_vrrp_start_obj = (
             amphora_driver_tasks.AmphoraIndexVRRPStart())
         mock_driver.start_vrrp_service.side_effect = [mock.DEFAULT,
                                                       Exception('boom')]
 
-        amphora_vrrp_start_obj.execute(0, [_amphora_mock],
+        amphora_vrrp_start_obj.execute(0, [_amphora_mock], amphorae_status,
+                                       _amphora_mock[constants.ID],
                                        timeout_dict=self.timeout_dict)
         mock_driver.start_vrrp_service.assert_called_once_with(
             _db_amphora_mock, self.timeout_dict)
 
+        # Unreachable amp
+        mock_driver.start_vrrp_service.reset_mock()
+        amphorae_status = {
+            _amphora_mock[constants.ID]: {
+                constants.UNREACHABLE: True
+            }
+        }
+        amphora_vrrp_start_obj.execute(0, [_amphora_mock], amphorae_status,
+                                       _amphora_mock[constants.ID],
+                                       timeout_dict=self.timeout_dict)
+        mock_driver.start_vrrp_service.assert_not_called()
+
         # Test with a start exception
         mock_driver.start_vrrp_service.reset_mock()
-        amphora_vrrp_start_obj.execute(0, [_amphora_mock],
+        amphora_vrrp_start_obj.execute(0, [_amphora_mock], {},
+                                       _amphora_mock[constants.ID],
                                        timeout_dict=self.timeout_dict)
         mock_driver.start_vrrp_service.assert_called_once_with(
             _db_amphora_mock, self.timeout_dict)
         mock_amphora_repo_update.assert_called_once_with(
             _session_mock, _db_amphora_mock.id, status=constants.ERROR)
 
+        # Test with a start exception, secondary amp
+        mock_driver.start_vrrp_service.reset_mock()
+        mock_amphora_repo_update.reset_mock()
+        amphora_vrrp_start_obj.execute(0, [_amphora_mock], {}, '1234',
+                                       timeout_dict=self.timeout_dict)
+        mock_driver.start_vrrp_service.assert_called_once_with(
+            _db_amphora_mock, self.timeout_dict)
+        mock_amphora_repo_update.assert_not_called()
+
     def test_amphora_compute_connectivity_wait(self,
                                                mock_driver,
                                                mock_generate_uuid,
@@ -1006,3 +1175,105 @@ class TestAmphoraDriverTasks(base.TestCase):
         self.assertRaises(driver_except.TimeOutException,
                           amp_config_update_obj.execute,
                           _amphora_mock, flavor)
+
+    def test_amphorae_get_connectivity_status(self,
+                                              mock_driver,
+                                              mock_generate_uuid,
+                                              mock_log,
+                                              mock_get_session,
+                                              mock_listener_repo_get,
+                                              mock_listener_repo_update,
+                                              mock_amphora_repo_get,
+                                              mock_amphora_repo_update):
+        amphora1_mock = mock.MagicMock()
+        amphora1_mock[constants.ID] = 'id1'
+        amphora2_mock = mock.MagicMock()
+        amphora2_mock[constants.ID] = 'id2'
+        db_amphora1_mock = mock.Mock()
+        db_amphora2_mock = mock.Mock()
+
+        amp_get_connectivity_status = (
+            amphora_driver_tasks.AmphoraeGetConnectivityStatus())
+
+        # All amphorae reachable
+        mock_amphora_repo_get.side_effect = [
+            db_amphora1_mock,
+            db_amphora2_mock]
+        mock_driver.check.return_value = None
+
+        ret = amp_get_connectivity_status.execute(
+            [amphora1_mock, amphora2_mock],
+            amphora1_mock[constants.ID],
+            timeout_dict=self.timeout_dict)
+        mock_driver.check.assert_has_calls(
+            [mock.call(db_amphora1_mock, timeout_dict=self.timeout_dict),
+             mock.call(db_amphora2_mock, timeout_dict=self.timeout_dict)])
+        self.assertFalse(
+            ret[amphora1_mock[constants.ID]][constants.UNREACHABLE])
+        self.assertFalse(
+            ret[amphora2_mock[constants.ID]][constants.UNREACHABLE])
+
+        # amphora1 unreachable
+        mock_driver.check.reset_mock()
+        mock_amphora_repo_get.side_effect = [
+            db_amphora1_mock,
+            db_amphora2_mock]
+        mock_driver.check.side_effect = [
+            driver_except.TimeOutException, None]
+        self.assertRaises(driver_except.TimeOutException,
+                          amp_get_connectivity_status.execute,
+                          [amphora1_mock, amphora2_mock],
+                          amphora1_mock[constants.ID],
+                          timeout_dict=self.timeout_dict)
+        mock_driver.check.assert_called_with(
+            db_amphora1_mock, timeout_dict=self.timeout_dict)
+
+        # amphora2 unreachable
+        mock_driver.check.reset_mock()
+        mock_amphora_repo_get.side_effect = [
+            db_amphora1_mock,
+            db_amphora2_mock]
+        mock_driver.check.side_effect = [
+            None, driver_except.TimeOutException]
+        ret = amp_get_connectivity_status.execute(
+            [amphora1_mock, amphora2_mock],
+            amphora1_mock[constants.ID],
+            timeout_dict=self.timeout_dict)
+        mock_driver.check.assert_has_calls(
+            [mock.call(db_amphora1_mock, timeout_dict=self.timeout_dict),
+             mock.call(db_amphora2_mock, timeout_dict=self.timeout_dict)])
+        self.assertFalse(
+            ret[amphora1_mock[constants.ID]][constants.UNREACHABLE])
+        self.assertTrue(
+            ret[amphora2_mock[constants.ID]][constants.UNREACHABLE])
+
+    def test_set_amphora_firewall_rules(self,
+                                        mock_driver,
+                                        mock_generate_uuid,
+                                        mock_log,
+                                        mock_get_session,
+                                        mock_listener_repo_get,
+                                        mock_listener_repo_update,
+                                        mock_amphora_repo_get,
+                                        mock_amphora_repo_update):
+        amphora = {constants.ID: AMP_ID, constants.VRRP_IP: '192.0.2.88'}
+        mock_amphora_repo_get.return_value = _db_amphora_mock
+
+        set_amp_fw_rules = amphora_driver_tasks.SetAmphoraFirewallRules()
+
+        # Test non-SRIOV VIP path
+        set_amp_fw_rules.execute([amphora], 0, [{'non-sriov-vip': True}], {},
+                                 timeout_dict=None)
+
+        mock_get_session.assert_not_called()
+        mock_driver.set_interface_rules.assert_not_called()
+
+        # Test SRIOV VIP path
+        set_amp_fw_rules.execute([amphora], 0, [{'fake_rule': True}], {},
+                                 timeout_dict=None)
+
+        mock_amphora_repo_get.assert_called_once_with(_session_mock, id=AMP_ID)
+
+        mock_driver.set_interface_rules.assert_called_once_with(
+            _db_amphora_mock, '192.0.2.88', [{'fake_rule': True}],
+            timeout_dict=None)
